# -*- coding: utf-8 -*-

import sys
import shutil
from decimal import Decimal
from django.db import models
from django.core.files.base import ContentFile
from djangoplus.utils.formatter import to_ascii
from djangoplus.ui.components.forms import fields as form_fields
from django.db.models.fields.files import ImageFieldFile
from djangoplus.utils.metadata import get_metadata, getattr2


# Superclass #


class FieldPlus(models.Field):
    def __init__(self, *args, **kwargs):
        self.search = kwargs.pop('search', False)
        self.filter = kwargs.pop('filter', False)
        self.exclude = kwargs.pop('exclude', False)
        self.example = kwargs.pop('example', False)
        self.display = kwargs.pop('display', True)
        self.formatter = kwargs.pop('formatter', None)
        if self.exclude and kwargs.get('default', None) is None:
            kwargs.update(null=True)
        super(FieldPlus, self).__init__(*args, **kwargs)


# Base Fields #

class AutoField(models.AutoField):
    pass


class CharField(models.CharField, FieldPlus):
    def __init__(self, *args, **kwargs):
        max_length = kwargs.pop('max_length', 255)
        super(CharField, self).__init__(*args, max_length=max_length, **kwargs)

    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.CharField)
        kwargs.setdefault('choices_form_class', form_fields.TypedChoiceField)
        field = super(CharField, self).formfield(**kwargs)
        return field


class EmailField(CharField):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.EmailField)
        return super(EmailField, self).formfield(**kwargs)


class UrlField(CharField):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.UrlField)
        return super(UrlField, self).formfield(**kwargs)


class AddressField(CharField):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.AddressField)
        return super(AddressField, self).formfield(**kwargs)


class TextField(models.TextField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.TextField)
        return super(TextField, self).formfield(**kwargs)


class FormattedTextField(models.TextField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.FormattedTextField)
        return super(FormattedTextField, self).formfield(**kwargs)


class IntegerField(models.IntegerField, FieldPlus):
    def __init__(self, *args, **kwargs):
        self.min_value = kwargs.pop('min_value', None)
        self.max_value = kwargs.pop('max_value', None)
        self.suffix = kwargs.pop('suffix', None)
        super(IntegerField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.IntegerField)
        kwargs.setdefault('choices_form_class', form_fields.TypedChoiceField)
        kwargs.setdefault('min_value', self.min_value)
        kwargs.setdefault('max_value', self.max_value)
        return super(IntegerField, self).formfield(**kwargs)


class DecimalField(models.DecimalField, FieldPlus):
    def __init__(self, *args, **kwargs):
        decimal_places = kwargs.pop('decimal_places', 2)
        max_digits = kwargs.pop('max_digits', 9)
        self.suffix = kwargs.pop('suffix', None)
        super(DecimalField, self).__init__(*args, decimal_places=decimal_places, max_digits=max_digits, **kwargs)

    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.DecimalField)
        return super(DecimalField, self).formfield(**kwargs)


class MoneyField(DecimalField):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.MoneyField)
        return super(MoneyField, self).formfield(**kwargs)


class DateField(models.DateField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.DateField)
        return super(DateField, self).formfield(**kwargs)


class CurrentDateField(DateField):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.CurrentDateField)
        return super(CurrentDateField, self).formfield(**kwargs)


class PastDateField(DateField):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.PastDateField)
        return super(PastDateField, self).formfield(**kwargs)


class FutureDateField(DateField):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.FutureDateField)
        return super(FutureDateField, self).formfield(**kwargs)


class DateTimeField(models.DateTimeField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.DateTimeField)
        return super(DateTimeField, self).formfield(**kwargs)


class CurrentDateTimeField(DateTimeField):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.CurrentDateTimeField)
        return super(CurrentDateTimeField, self).formfield(**kwargs)


class PastDateTimeField(DateTimeField):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.PastDateTimeField)
        return super(PastDateTimeField, self).formfield(**kwargs)


class FutureDateTimeField(DateTimeField):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.FutureDateTimeField)
        return super(FutureDateTimeField, self).formfield(**kwargs)


class BooleanField(models.BooleanField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.BooleanField)
        return super(BooleanField, self).formfield(**kwargs)


class NullBooleanField(models.NullBooleanField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.NullBooleanField)
        return super(NullBooleanField, self).formfield(**kwargs)


# One-to-One Relation Field #

class OneToOneField(models.OneToOneField, FieldPlus):

    def __init__(self, *args, **kwargs):
        kwargs.update(on_delete=models.SET_NULL)
        super(OneToOneField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.OneToOneField)
        return super(OneToOneField, self).formfield(**kwargs)


# Model Choice Field #

class ForeignKey(models.ForeignKey, FieldPlus):
    def __init__(self, *args, **kwargs):
        self.lazy = kwargs.pop('lazy', False)
        self.ignore_lookup = kwargs.pop('ignore_lookup', False)
        self.form_filter = kwargs.pop('form_filter', None)
        self.composition = kwargs.pop('composition', False)
        self.tree = kwargs.pop('tree', False)
        if self.tree:
            kwargs.update(null=True)
            kwargs.update(blank=True)
        kwargs.update(on_delete=models.CASCADE)
        if self.form_filter and type(self.form_filter) not in (tuple, list):
            self.form_filter = self.form_filter.split('__')[-1], self.form_filter
        self.queryset_filter = kwargs.pop('queryset_filter', None)
        super(ForeignKey, self).__init__(*args, **kwargs)
        if 'test' in sys.argv:
            self.form_filter = None

    def formfield(self, **kwargs):

        kwargs.setdefault('lazy', self.lazy)
        kwargs.setdefault('ignore_lookup', self.ignore_lookup)
        kwargs.setdefault('form_filters', self.form_filter and [self.form_filter] or [])
        kwargs.setdefault('form_class', form_fields.ModelChoiceField)

        field = super(ForeignKey, self).formfield(**kwargs)
        if self.queryset_filter:
            field.queryset = field.queryset.filter(**self.queryset_filter)
        return field


ModelChoiceField = ForeignKey


# Multiple Model Choice Fields #

class ManyToManyField(models.ManyToManyField, FieldPlus):
    def __init__(self, *args, **kwargs):
        self.lazy = kwargs.pop('lazy', False)
        self.ignore_lookup = kwargs.pop('ignore_lookup', False)
        self.form_filter = kwargs.pop('form_filter', None)
        if self.form_filter and type(self.form_filter) not in (tuple, list):
            self.form_filter = self.form_filter.split('__')[-1], self.form_filter
        self.queryset_filter = kwargs.pop('queryset_filter', None)
        super(ManyToManyField, self).__init__(*args, **kwargs)
        if 'test' in sys.argv:
            self.form_filter = None

    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.MultipleModelChoiceField)
        kwargs.setdefault('ignore_lookup', self.ignore_lookup)
        kwargs.setdefault('form_filters', self.form_filter and [self.form_filter] or [])
        kwargs.setdefault('lazy', self.lazy)
        field = super(ManyToManyField, self).formfield(**kwargs)
        if self.queryset_filter:
            field.queryset = field.queryset.filter(**self.queryset_filter)
        return field

MultipleModelChoiceField = ManyToManyField


class OneToManyField(ManyToManyField):

    def __init__(self, *args, **kwargs):
        self.one_to_many_count = kwargs.pop('count', None)
        super(OneToManyField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.OneToManyField)
        form_field = super(OneToManyField, self).formfield(**kwargs)
        form_field.one_to_many_count = self.one_to_many_count
        return form_field

# File Fields #

class FileField(models.FileField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.FileField)
        return super(FileField, self).formfield(**kwargs)


class ImageFieldFile(ImageFieldFile):
    def __init__(self, *args, **kwargs):
        super(ImageFieldFile, self).__init__(*args, **kwargs)

        if self.field.sizes:
            def get_size(self, size):
                if not self:
                    return ''
                else:
                    split = self.url.rsplit('.', 1)
                    thumb_url = '{}.{}x{}.{}'.format(split[0], w, h, split[1])
                    return thumb_url

            first = True
            for size in self.field.sizes:
                (w, h) = size
                if first:
                    first = False
                    if self:
                        setattr(self, 'url_{}x{}'.format(w, h), self.url)
                        setattr(self, 'small', self.url)
                    else:
                        setattr(self, 'url_{}x{}'.format(w, h), '')
                        setattr(self, 'small', '')
                else:
                    setattr(self, 'url_{}x{}'.format(w, h), get_size(self, size))
                    setattr(self, 'large', get_size(self, size))

    def generate_thumb(self, img, thumb_size, file_format):
        # return img
        import io
        from PIL import Image
        img.seek(0)  # see http://code.djangoproject.com/ticket/8222 for details
        image = Image.open(img)

        # Convert to RGB if necessary
        if image.mode not in ('L', 'RGB', 'RGBA'):
            image = image.convert('RGB')

        # get size
        thumb_w, thumb_h = thumb_size
        # If you want to generate a square thumbnail
        if thumb_w == thumb_h and False:
            # quad
            xsize, ysize = image.size
            # get minimum size
            minsize = min(xsize, ysize)
            # largest square possible in the image
            xnewsize = (xsize - minsize) / 2
            ynewsize = (ysize - minsize) / 2
            # crop it
            image2 = image.crop((xnewsize, ynewsize, xsize - xnewsize, ysize - ynewsize))
            # load is necessary after crop                
            image2.load()
            # thumbnail of the cropped image (with ANTIALIAS to make it look better)
            image2.thumbnail(thumb_size, Image.ANTIALIAS)
        else:
            # not quad
            image2 = image
            image2.thumbnail(thumb_size, Image.ANTIALIAS)

        io = io.StringIO()
        # PNG and GIF are the same, JPG is JPEG
        if file_format.upper() == 'JPG':
            file_format = 'JPEG'

        image2.save(io, file_format)
        return ContentFile(io.getvalue())

    def save(self, name, content, save=True):
        super(ImageFieldFile, self).save(name, content, save)

        first = True
        if self.field.sizes:
            for size in self.field.sizes:
                (w, h) = size
                split = self.name.rsplit('.', 1)
                thumb_name = '{}.{}x{}.{}'.format(split[0], w, h, split[1])

                # you can use another thumbnailing function if you like
                thumb_content = self.generate_thumb(content, size, split[1])

                thumb_name_ = self.storage.save(thumb_name, thumb_content)

                if not thumb_name == thumb_name_:
                    raise ValueError('There is already a file named {}'.format(thumb_name))

                if first:
                    first = False
                    shutil.move(self.path.replace(self.name, thumb_name), self.path)

    def delete(self, save=True):
        name = self.name
        super(ImageFieldFile, self).delete(save)
        if self.field.sizes:
            for size in self.field.sizes:
                (w, h) = size
                split = name.rsplit('.', 1)
                thumb_name = '{}.{}x{}.{}'.format(split[0], w, h, split[1])
                try:
                    self.storage.delete(thumb_name)
                except:
                    pass


class ImageField(models.ImageField, FieldPlus):
    attr_class = ImageFieldFile

    def __init__(self, verbose_name=None, name=None, width_field=None, height_field=None, sizes=None, **kwargs):
        self.verbose_name = verbose_name
        self.name = name
        self.width_field = width_field
        self.height_field = height_field
        self.sizes = sizes
        default = '/static/images/blank.png'
        if 'default' in kwargs:
            default = kwargs.pop('default')
        super(ImageField, self).__init__(default=default, verbose_name=self.verbose_name, **kwargs)

    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.ImageField)
        return super(ImageField, self).formfield(**kwargs)


class PhotoField(models.ImageField, FieldPlus):
    attr_class = ImageFieldFile

    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.PhotoField)
        return super(PhotoField, self).formfield(**kwargs)


# Regional Fields #

class CpfField(CharField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.CpfField)
        return super(CpfField, self).formfield(**kwargs)


class CnpjField(CharField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.CnpjField)
        return super(CnpjField, self).formfield(**kwargs)


class CpfCnpjField(CharField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.CpfCnpjField)
        return super(CpfCnpjField, self).formfield(**kwargs)


class CepField(CharField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.CepField)
        return super(CepField, self).formfield(**kwargs)


class CarPlateField(CharField, FieldPlus):
    def __init__(self, **kwargs):
        super(CarPlateField, self).__init__(**kwargs)

    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.CarPlateField)
        return super(CarPlateField, self).formfield(**kwargs)


class PhoneField(CharField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.PhoneField)
        return super(PhoneField, self).formfield(**kwargs)


# Utilitary Fields #

class SearchField(TextField):
    def pre_save(self, model_instance, add):
        search_text = []
        for attr_name in get_metadata(model_instance, 'search_fields'):
            tokens = attr_name.split('__')
            if len(tokens) == 1:
                if attr_name != 'ascii':
                    val = getattr2(model_instance, attr_name)
                    if val:
                        search_text.append(str(to_ascii(val).upper().strip()))
        return ' '.join(search_text)


class HtmlTextField(models.TextField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.TextField)
        return super(HtmlTextField, self).formfield(**kwargs)

    def to_python(self, value):
        return None


class PasswordField(models.CharField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.PasswordField)
        return super(PasswordField, self).formfield(**kwargs)


class Decimal3(Decimal):
    def __new__(cls, value="0", context=None):
        cls.decimal3 = True
        return Decimal.__new__(cls, value=value or "0", context=context)


class DecimalField3(models.DecimalField, FieldPlus):
    def __init__(self, *args, **kwargs):
        decimal_places = kwargs.pop('decimal_places', 3)
        max_digits = kwargs.pop('max_digits', 9)
        super(DecimalField3, self).__init__(*args, decimal_places=decimal_places, max_digits=max_digits, **kwargs)

    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.DecimalField3)
        return super(DecimalField3, self).formfield(**kwargs)

    def from_db_value(self, value, expression, connection, context):
        value = Decimal3(value)
        return value


class CreditCardField(CharField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.CreditCardField)
        return super(CreditCardField, self).formfield(**kwargs)


class OneDigitValidationField(CharField, FieldPlus):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.OneDigitValidationField)
        return super(OneDigitValidationField, self).formfield(**kwargs)


class CurrentUserField(models.ForeignKey, FieldPlus):
    def __init__(self, *args, **kwargs):
        kwargs['to'] = 'admin.User'
        kwargs['null'] = True
        kwargs['blank'] = True
        super(CurrentUserField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.CurrentUserField)
        return super(CurrentUserField, self).formfield(**kwargs)


class PositiveIntegerField(models.PositiveIntegerField, FieldPlus):
    def __init__(self, *args, **kwargs):
        self.min_value = kwargs.pop('min_value', 0)
        self.max_value = kwargs.pop('max_value', None)
        self.suffix = kwargs.pop('suffix', None)
        super(PositiveIntegerField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', form_fields.IntegerField)
        kwargs.setdefault('min_value', self.min_value)
        kwargs.setdefault('max_value', self.max_value)
        return super(PositiveIntegerField, self).formfield(**kwargs)


class TreeIndexField(CharField):
    def __init__(self, *args, **kwargs):
        self.ref = kwargs.pop('ref', None)
        self.sep = kwargs.pop('sep', '.')
        if not self.ref:
            kwargs.update(display=None)
        kwargs.update(blank=True, exclude=True, null=True)
        super(TreeIndexField, self).__init__(*args, **kwargs)



# -*- coding: utf-8 -*-

import json
from django.apps import apps
from django.conf import settings
from djangoplus.docs import utils
from djangoplus.cache import loader
from django.http import HttpRequest
from django.utils import translation
from djangoplus.utils import terminal
from djangoplus.admin.models import User
from django.utils.translation import ugettext as _
from django.template.defaultfilters import slugify
from djangoplus.ui.components.forms import factory, fields as form_fields
from djangoplus.db.models import fields as model_fields
from djangoplus.utils.metadata import get_metadata, find_action, find_model_by_verbose_name, find_model_by_add_label, get_field, find_model_by_verbose_name_plural, find_subset_by_title


class Actor(object):
    def __init__(self, name=None, description=None, scope=None):
        self.name = name
        self.description = description
        self.scope = scope

    def __str__(self):
        return self.name

    def as_dict(self):
        return dict(name=self.name, description=self.description, scope=self.scope)


class UseCase(object):
    
    ACTOR = None

    """ This class has the aim of generating interactions and testcases based on usecases.
    The code generation is based on strings that holds the usercase name.
    The usecases must follow the sintaxe bellow:
        Login as <group name>
        Register <class verbose name>
        List <class plural verbose name>
        List <class plural verbose name>: <subset name>
        Add <relation name> to <class verbose name>
        <action name> in <class verbose name>
        <action name>
    Code generation is also compatible with Porguese and may also follow the sintaxe bellow
        Acessar como <nome do grupo>
        Cadastrar <nome amigável da classe>
        Listar <nome amigável da classe no plural>
        Listar <nome amigável da classe no plural>: <nome do subconjunto>
        Adicionar <nome da relação> em <nome amigável da classe>
        <nome da ação> em <nome amigável da classe>
        <nome da ação>
    """

    def __init__(self, name):

        translation.activate(settings.LANGUAGE_CODE)

        self._interactions = []

        self._test_flow_code = []
        self._test_function_code = []

        self.name = name
        self.description = None
        self.actors = []
        self.business_rules = []
        self.pre_conditions = []
        self.post_condition = None

        self._request = None

        if name.startswith(_('Access')):
            self._login(name)
        elif name.startswith(_('List')):
            self._list(name)
        elif name.startswith(_('Register')) and _('in') not in name:
            self._register(name)
        elif name.startswith(_('Add')):
            self._add(name)
        else:
            self._execute(name)

    def _mocked_request(self):
        if not self._request:
            self._request = HttpRequest()
            self._request.user = User(pk=0)
        return self._request

    def _debug(self):
        print('\n'.join(self._interactions))
        print()
        print('\n'.join(self._test_function_code))
        print()
        print('\n'.join(self._test_flow_code))

    def _login(self, action):
        verbose_name = action.split(_('as'))[-1].strip()
        loader.last_authenticated_role = verbose_name
        model = find_model_by_verbose_name(verbose_name)
        if model:
            role_username = get_metadata(model, 'role_username')
            if role_username:
                field = get_field(model, role_username)
                username = field.example or ''
            else:
                username = settings.DEFAULT_SUPERUSER
        else:
            username = username = settings.DEFAULT_SUPERUSER
        password = settings.DEFAULT_PASSWORD

        interaction = _('The user access the system as')
        self._interactions.append('{} {}'.format(interaction, verbose_name))
        self._interactions.append(_('The system displays the main page'))

        self._test_flow_code.append('\t\t# {}'.format(action))
        self._test_flow_code.append("\t\tif self.login('{}', '{}'):".format(username, password))

    def _find(self, model):
        click_str = []
        click_str_unicode = []

        verbose_name = get_metadata(model, 'verbose_name')
        verbose_name_plural = get_metadata(model, 'verbose_name_plural')
        list_shortcut = get_metadata(model, 'list_shortcut', [], iterable=True)
        list_menu = get_metadata(model, 'list_menu')

        if not loader.last_authenticated_role or True in list_shortcut or loader.last_authenticated_role in list_shortcut:

            left = _('The user clicks on the shortcut card')
            right = _('in the dashboard at main page')
            interaction = '{} "{}" {}'.format(left, verbose_name_plural, right)
            self._interactions.append(interaction)

            self._interactions.append(_('The system displays the listing page'))

            self._test_function_code.append('\t\tself.click_link(u\'{}\')'.format(verbose_name_plural))

        elif list_menu:

            if type(list_menu) == tuple:
                list_menu = list_menu[0]
            for menu_item in list_menu.split('::'):
                click_str.append('"{}"'.format(menu_item))
                click_str_unicode.append("'{}'".format(menu_item))

            interaction = _('The user access the menu')
            self._interactions.append('{} {}'.format(interaction, ', '.join(click_str)))

            self._test_function_code.append('\t\tself.click_menu({})'.format(', '.join(click_str_unicode)))

        else:
            raise ValueError('There is no way to access the model "{}", please do one of the following things:\n '
                             'i) Add the "list_menu" meta-attribute to model\n '
                             'ii) Add the "list_shortcut" meta-attribute to model\n '
                             'iii) Add the "composotion" field attribute to one of foreignkey fields of the model if '
                             'it exists and the the relation on the foreignkey model'.format(verbose_name))

    def _view(self, model, recursive=False):

        list_shortcut = get_metadata(model, 'list_shortcut', [])
        list_menu = get_metadata(model, 'list_menu')

        # if the model can be accessed by the menu or shortcut
        if list_shortcut or list_menu:
            self._find(model)
            self._interactions.append(_('The user locates the record and clicks the visualization icon'))
            self._interactions.append(_('The system displays the visualization page'))
            self._test_function_code.append("\t\tself.click_icon('{}')".format('Visualizar'))  # _('Visualize')
        else:
            # the model can be accesses only by a parent model
            for parent_model in loader.composition_relations:
                if model in loader.composition_relations[parent_model]:
                    self._view(parent_model, True)
                    panel_title = None
                    if hasattr(parent_model, 'fieldsets'):
                        for fieldset in parent_model.fieldsets:
                            if 'relations' in fieldset[1]:
                                for item in fieldset[1]['relations']:
                                    relation = getattr(parent_model, item)
                                    if hasattr(relation, 'field') and relation.field.remote_field.model == model:
                                        panel_title = fieldset[0]
                                        break
                    if panel_title:
                        if '::' in panel_title:
                            tab_name, panel_title = panel_title.split('::')
                            self._test_function_code.append("\t\tself.click_tab('{}')".format(tab_name))
                            interaction = _('The user clicks the tab')
                            self._interactions.append('{} "{}"'.format(interaction, tab_name))
                        if panel_title:
                            interaction = _('The user looks at the painel')
                            self._interactions.append('{} "{}"'.format(interaction, panel_title))
                            self._test_function_code.append("\t\tself.look_at_panel('{}')".format(panel_title))

                    if recursive:
                        self._interactions.append(_('The user locates the record and clicks the visualization icon'))
                        self._interactions.append(_('The system displays the visualization page'))
                        self._test_function_code.append("\t\tself.click_icon('{}')".format('Visualizar'))  # _('Visualize')

    def _list(self, action):
        verbose_name_plural = action.replace(_('List'), '').strip()

        if ':' in action:
            # it refers to a subset
            verbose_name_plural, subset_name = verbose_name_plural.split(':')
            verbose_name_plural = verbose_name_plural.strip()
            model = find_model_by_verbose_name_plural(verbose_name_plural)
            subset_name = subset_name.strip()
            subset = find_subset_by_title(subset_name, model)

            # set the attributes of the usecase
            self.description = subset['function'].__doc__.strip()
            for can_view in subset['can_view']:
                self.actors.append(can_view)
            if not self.actors:
                self.actors.append('Superusuário')

            # register the interactions and testing code
            self._find(model)

            interaction = _('The user clicks the pill')
            self._interactions.append('{} "{}"'.format(interaction, subset_name))

            self._test_function_code.append("\t\tself.click_link('{}')".format(subset_name))

        else:
            model = find_model_by_verbose_name_plural(verbose_name_plural)

            # set the attributes of the usecase
            self.description = 'List {} previouly registered in the system'.format(verbose_name_plural.lower())
            self.post_condition = _('The system displays the registered records')
            for meta_data in ('can_admin', 'can_admin_by_role', 'can_admin_by_unit', 'can_admin_by_organization',
                              'can_view', 'can_view_by_role', 'can_view_by_unit', 'can_view_by_organization'):
                for actor in get_metadata(model, meta_data, iterable=True):
                    if actor:
                        self.actors.append(actor)
            if not self.actors:
                self.actors.append('Superusuário')

            # register the interactions and testing code
            self._find(model)

        # register the interactions
        search_fields = utils.get_search_fields(model)
        if search_fields:
            interaction = _('The user optionally performs a search by')
            self._interactions.append('{} {}'.format(interaction, search_fields))

        list_filter = utils.get_list_filter(model)
        if list_filter:
            interaction = _('The user optionally filter the records by')
            self._interactions.append('{} {}'.format(interaction, list_filter))

        left = _('The system displays')
        right = _('of previous registered records')
        interaction = '{} {} {}'.format(left, utils.get_list_display(model), right)
        self._interactions.append(interaction)

    def _register(self, action):
        model = find_model_by_add_label(action)
        if model:
            verbose_name = get_metadata(model, 'verbose_name')
            button_label = get_metadata(model, 'add_label')
            func_name = slugify(button_label).replace('-', '_')
        else:
            verbose_name = action.replace(_('Register'), '').strip()
            model = find_model_by_verbose_name(verbose_name)
            button_label = _('Register')
            func_name = 'cadastrar_{}'.format(model.__name__.lower())

        # set the attributes of the usecase
        self.name = action
        self.description = '{} {} {}'.format(_('Add new records of'),  verbose_name.lower(), _('in the system'))
        self.business_rules = utils.extract_exception_messages(model.save)
        self.post_condition = _('The record will be successfully registered in the system')
        required_data = []
        for field in get_metadata(model, 'get_fields'):
            if isinstance(field, model_fields.ForeignKey):
                if not isinstance(field, model_fields.OneToOneField):
                    if not isinstance(field, model_fields.OneToManyField):
                        required_data.append(field.verbose_name.lower())
        if required_data:
            pre_condition = _('The following information must have been previouly registered in the system: ')
            self.pre_conditions.append('{} {}'.format(pre_condition, ', '.join(required_data)))

        for meta_data in ('can_admin', 'can_admin_by_role', 'can_admin_by_unit', 'can_admin_by_organization', 'can_add',
                          'can_add_by_role', 'can_add_by_unit', 'can_add_by_organization'):
            for actor in get_metadata(model, meta_data, iterable=True):
                if actor:
                    self.actors.append(actor)
        if not self.actors:
            self.actors.append(_('Superuser'))

        # register the interactions and testing code
        func_signature = '{}(self)'.format(func_name)
        self._test_flow_code.append('\t\t\t# {}'.format(action))
        self._test_flow_code.append('\t\t\tself.{}()'.format(func_name))

        self._test_function_code.append('\tdef {}:'.format(func_signature))
        self._find(model)
        self._test_function_code.append("\t\tself.click_button('{}')".format(button_label))

        a = _('The user clicks the button')
        b = _('on the right-side of the action bar')
        self._interactions.append('{} "{}" {}'.format(a, button_label, b))

        form = factory.get_register_form(self._mocked_request(), model())
        self._fill_out(form)

        interaction = _('The user clicks the button')
        self._interactions.append('{} "{}"'.format(interaction, button_label))
        self._test_function_code.append("\t\tself.click_button('{}')".format(button_label))
        self._test_function_code.append("\t\tself.click_icon('{}')".format('Principal'))

    def _add(self, action):

        model = None

        if action.startswith(_('Add')):
            # not add_label was defined for the related model
            tokens = action.replace(_('Add'), '').split(_(' in '))
            verbose_name = tokens[0].strip()
            if len(tokens) > 1:
                relation_verbose_name = tokens[1].strip()
                model = find_model_by_verbose_name(relation_verbose_name)
            related_model = find_model_by_verbose_name(verbose_name)
        else:
            # an add_label was defined for the related model
            tokens = action.split(_(' in '))
            add_label = tokens[0].strip()
            if len(tokens) > 1:
                verbose_name = tokens[1].strip()
                model = find_model_by_verbose_name(verbose_name)
            related_model = find_model_by_add_label(add_label)

        # check if there is a fieldset was defined with the relation
        relation_name = None
        inlines = []
        if hasattr(model, 'fieldsets'):
            for fieldset in model.fieldsets:
                if 'relations' in fieldset[1]:
                    for item in fieldset[1]['relations']:
                        tmp = getattr(model, item)
                        if hasattr(tmp, 'field') and tmp.field.remote_field.model == model:
                            relation_name = item
                if 'inlines' in fieldset[1]:
                    for item in fieldset[1]['inlines']:
                        inlines.append(item)
                        tmp = getattr(model, item)
                        if hasattr(tmp, 'field') and tmp.field.remote_field.model == model:
                            relation_name = item

        # if the relation was defined in a fieldset
        if relation_name:
            add_inline = relation_name in inlines
            add_label = get_metadata(related_model, 'add_label')
            button_label = add_label or 'Adicionar'
            button_label = get_metadata(related_model, 'add_label', button_label)

            function_signature = '{}_{}_{}_{}'.format(_('add'), related_model.__name__.lower(), _('in'), model.__name__.lower())
            self._test_flow_code.append('\t\t\t# {}'.format(action))
            self._test_flow_code.append('\t\t\tself.{}()'.format(function_signature))

            self._test_function_code.append('\tdef {}(self):'.format(function_signature))

            self._view(related_model)

            # the form is not in the visualization page and it must be opened
            if not add_inline:
                add_button_label = 'Adicionar {}'.format(get_metadata(related_model, 'verbose_name'))
                add_button_label = get_metadata(related_model, 'add_label', add_button_label)

                interaction = _('The user clicks the button')
                self._interactions.append('{} "{}"'.format(interaction, add_button_label))
                self._interactions.append(_('The system displays a popup window'))

                self._test_function_code.append("\t\tself.click_button('{}')".format(add_button_label))
                self._test_function_code.append("\t\tself.look_at_popup_window()")

            form = factory.get_many_to_one_form(self._mocked_request(), model(), relation_name, related_model())
            self._fill_out(form)

            interaction = _('The user clicks the button')
            self._interactions.append('{} "{}"'.format(interaction, button_label))

            self._test_function_code.append("\t\tself.click_button('{}')".format(button_label))
            self._test_function_code.append("\t\tself.click_icon('{}')".format('Principal'))
        else:
            raise ValueError('Please add the {}\'s relation in the fieldsets of model {}'.format(related_model.__name__, model.__name__))

    def _fill_out(self, form, inline=None):

        if not inline:
            self._interactions.append(_('The system displays the input form'))

        form.contextualize()
        form.configure()

        for fieldset in form.configured_fieldsets:
            if inline or fieldset['title']:
                if fieldset['tuples'] and fieldset['tuples'][0]:
                    interaction = _('The user looks at fieldset')
                    self._interactions.append('{} "{}"'.format(interaction, inline or fieldset['title']))

            for fields in fieldset['tuples']:
                if not type(fields) in (list, tuple):
                    fields = fields,
                field_names = [field['name'] for field in fields]

                for field_name in field_names:
                    if field_name in form.base_fields:
                        form_field = form.base_fields[field_name]

                        # it is necessary to skip "organization" and "unit" fields if the actor's scope does not match
                        if self.actors and hasattr(form_field, 'queryset'):
                            from djangoplus.admin.models import Organization, Unit
                            scopes = []
                            if issubclass(form_field.queryset.model, Organization):
                                scopes = ['systemic']
                            if issubclass(form_field.queryset.model, Unit):
                                scopes = ['systemic', 'organization']
                            if scopes:
                                ignore_field = False
                                for role_model in loader.role_models:
                                    if loader.role_models[role_model]['name'] == self.actors[0]:
                                        if loader.role_models[role_model]['scope'] not in scopes:
                                            ignore_field = True
                                            break
                                if ignore_field:
                                    continue

                        # get the test value in case of model form and if the sample value was defined in the model
                        value = ''
                        meta_class = getattr(form, '_meta', None)
                        if meta_class:
                            if hasattr(meta_class.model, field_name):
                                model_field = get_field(meta_class.model, field_name)
                                if hasattr(model_field, 'example') and model_field.example:
                                    value = callable(model_field.example) and model_field.example() or model_field.example

                        # define the kind of user interaction
                        if hasattr(form_field, 'choices') and form_field.choices:
                            self._test_function_code.append("\t\tself.choose('{}', '{}')".format(form_field.label, value))
                            interaction = _('The user chooses')
                            self._interactions.append('{} "{}"'.format(interaction, form_field.label))
                        else:
                            if not isinstance(form_field, form_fields.BooleanField) and not isinstance(form_field, form_fields.NullBooleanField):
                                self._test_function_code.append("\t\tself.enter('{}', '{}')".format(form_field.label, value))
                                interaction = _('The user enters')
                                self._interactions.append('{} "{}"'.format(interaction, form_field.label))

            # add the interactions recursively if the form inner forms
            for field, form, required, save in fieldset.get('one_to_one', ()):
                self._fill_out(form, inline=field.label)
            for field, one_to_many_forms in fieldset.get('one_to_many', ()):
                for form in one_to_many_forms[0:1]:
                    self._fill_out(form, inline=field.label)

    def _execute(self, action):
        tokens = action.split(_(' in '))
        if find_model_by_add_label(tokens[0].strip()):
            if len(tokens) == 1:
                self._register(action)
            else:
                self._add(action)
        else:
            if len(tokens) == 1:
                self._execute_view(action)
            else:
                self._execute_action(action)

    def _execute_action(self, action):
        tokens = action.split(_(' in '))
        action_name = tokens[0].strip()
        verbose_name = tokens[1].strip()
        model = find_model_by_verbose_name(verbose_name)
        action_dict = find_action(model, action_name)
        func = action_dict['function']

        func_signature = '{}_em_{}(self)'.format(func.__name__.lower(), model.__name__.lower())
        self._test_flow_code.append('\t\t\t# {}'.format(action))
        self._test_flow_code.append('\t\t\tself.{}_em_{}()'.format(func.__name__.lower(), model.__name__.lower()))

        self._test_function_code.append('\tdef {}:'.format(func_signature))
        self._view(model, True)
        if hasattr(func, '_action'):
            button_label = func._action['title']
            params = func.__code__.co_varnames[1:func.__code__.co_argcount]
            if params:
                interaction = _('The user clicks the button')
                self._interactions.append('{} "{}"'.format(interaction, button_label))
                self._interactions.append(_('The system displays a popup window'))

                self._test_function_code.append("\t\tself.click_button('{}')".format(button_label))
                self._test_function_code.append("\t\tself.look_at_popup_window()")
                form = factory.get_action_form(self._mocked_request(), model(), func._action)
                self._fill_out(form)

            interaction = _('The user clicks the button')
            self._interactions.append('{} "{}"'.format(interaction, button_label))
            self._test_function_code.append("\t\tself.click_button('{}')".format(button_label))
            self._test_function_code.append("\t\tself.click_icon('{}')".format('Principal'))

            description = func.__doc__ and func.__doc__.strip() or ''
            business_rules = utils.extract_exception_messages(func)
            post_condition = _('Action successfully performed')
            self.name = action_dict['title']
            self.description = description
            self.business_rules = business_rules
            self.post_condition = post_condition
            for can_execute in action_dict['can_execute']:
                self.actors.append(can_execute)
            if not self.actors:
                self.actors.append(_('Superuser'))

    def _execute_view(self, action):
        pass

    def get_interactions_as_string(self):
        l = []
        for i, interaction in enumerate(self._interactions):
            l.append('\t\t\t{}. {}'.format(i+1, interaction))
        return '\n'.join(l)

    def get_test_flow_code(self):
        return self._test_flow_code

    def get_test_function_code(self):
        return self._test_function_code

    def as_dict(self):
        return dict(name=self.name, description=self.description, actors=self.actors,
                    business_rules=self.business_rules, pre_conditions=self.pre_conditions,
                    post_condition=self.post_condition, scenario=self.get_interactions_as_string())

    def __str__(self):
        l = list()
        l.append('')
        l.append('{}:\t\t\t{}'.format(terminal.info(_('Name')), self.name))
        l.append('{}:\t\t{}'.format(terminal.info(_('Description')), self.description or ''))
        l.append('{}:\t\t\t{}'.format(terminal.info(_('Actors')), ', '.join(self.actors)))
        l.append('{}:\t\t{}'.format(terminal.info(_('Buniness Rules')), ', '.join(self.business_rules)))
        l.append('{}:\t\t{}'.format(terminal.info(_('Pre-conditions')), ', '.join(self.pre_conditions)))
        l.append('{}:\t\t{}'.format(terminal.info(_('Post-condition')), (self.post_condition or '')))
        l.append('{}:'.format(terminal.info(_('Main-scenario'))))
        l.append(self.get_interactions_as_string())
        l.append('')
        return '\n'.join(l)


class Workflow(object):

    def __init__(self):

        translation.activate(settings.LANGUAGE_CODE)

        self.actors = []
        self.tasks = []
        tmp = None
        for task in loader.workflows:
            role = task['role']
            activity = task['activity']
            model = task['model']

            if role != tmp:
                tmp = role or _('Superuser')
                action = _('Acessar como')
                self.tasks.append('{} {}'.format(action, tmp))

            if model:
                action = '{}{}{}'.format(activity, _(' in '), model)
            else:
                action = activity
            self.tasks.append(action)


class ClassDiagram(object):

    POSITION_MAP = {
        1: (2.2,), 2: (1.2, 3.2,), 3: (3.2, 1.1, 1.3,), 4: (2.2, 3.2, 1.1, 1.3,), 5: (2.2, 1.1, 1.3, 3.1, 3.3,),
        6: (1.2, 3.2, 1.1, 3.1, 1.3, 3.3,), 7: (2.2, 1.2, 3.2, 1.1, 1.3, 3.1, 3.3,),
        8: (1.2, 3.2, 1.1, 1.3, 2.1, 2.3, 3.1, 3.3,), 9: (1.1, 1.2, 1.3, 2.1, 2.2, 2.3, 3.1, 3.2, 3.3,)
    }

    def __init__(self, class_diagram_name, models):

        self.name = class_diagram_name
        self.classes = []
        self.compositions = []
        self.agregations = []

        classes = dict()
        n = len(loader.class_diagrams[class_diagram_name])
        associations_count = {}
        for model in loader.class_diagrams[class_diagram_name]:
            associations_count[model] = 0

        for model in loader.class_diagrams[class_diagram_name]:
            verbose_name = get_metadata(model, 'verbose_name')
            related_objects = get_metadata(model, 'related_objects')
            classes[model] = dict(name=verbose_name, position='1.1')
            for related_object in related_objects:
                related_verbose_name = get_metadata(related_object.related_model, 'verbose_name')
                if related_object.related_model in models:
                    if hasattr(related_object.field, 'composition') and related_object.field.composition:
                        self.compositions.append(
                            [related_verbose_name, verbose_name, related_object.remote_field.name])
                        associations_count[model] += 1
                        associations_count[related_object.related_model] += 1
                    else:
                        if (model not in loader.role_models or class_diagram_name == related_verbose_name) or (
                                        class_diagram_name == verbose_name and related_object.related_model not in loader.role_models):
                            self.agregations.append(
                                [verbose_name, related_verbose_name, related_object.field.name])
                            associations_count[related_object.related_model] += 1
                            associations_count[model] += 1
        sorted_associations_count = sorted(associations_count, key=associations_count.get, reverse=True)
        for i, model in enumerate(sorted_associations_count):
            cls = classes[model]
            cls['position'] = ClassDiagram.POSITION_MAP[n][i]
            self.classes.append(cls)

    def as_dict(self):
        return dict(name=self.name, classes=self.classes, compositions=self.compositions, agregations=self.agregations)

    def as_json(self):
        return json.dumps(self.as_dict())


class Documentation(object):

    def __init__(self):

        translation.activate(settings.LANGUAGE_CODE)

        self.description = None
        self.workflow = None
        self.actors = []
        self.usecases = []
        self.class_diagrams = []

        # load description
        for app_config in apps.get_app_configs():
            if app_config.label == settings.PROJECT_NAME:
                self.description = app_config.module.__doc__ and app_config.module.__doc__.strip() or None

        # load actors
        organization_name = loader.organization_model and get_metadata(loader.organization_model, 'verbose_name') or None
        unit_name = loader.unit_model and get_metadata(loader.unit_model, 'verbose_name') or None

        for model in loader.role_models:

            name = loader.role_models[model]['name']
            description = utils.extract_documentation(model)
            scope = ''
            if organization_name:
                scope = loader.role_models[model]['scope']
                if scope == 'systemic':
                    scope = _('Systemic')
                elif scope == 'organization':
                    scope = organization_name
                elif scope == 'unit':
                    scope = unit_name
            self.actors.append(Actor(name=name, scope=scope, description=description))

        # load usecases
        self.workflow = Workflow()
        for task in self.workflow.tasks:
            if not task.startswith(_('Access')):
                usecase = UseCase(task)
                self.usecases.append(usecase)

        # load class diagrams
        for class_diagram_name, models in list(loader.class_diagrams.items()):
            class_diagram = ClassDiagram(class_diagram_name, models)
            self.class_diagrams.append(class_diagram)

    def as_dict(self):
        return dict(description=self.description, actors=[actor.as_dict() for actor in self.actors],
                    usecases=[usecase.as_dict() for usecase in self.usecases],
                    class_diagrams=[class_diagram.as_json() for class_diagram in self.class_diagrams])

    def as_json(self):
        return json.dumps(self.as_dict())

    def __str__(self):
        l = list()
        l.append(terminal.bold(_('Description:').upper()))
        l.append(terminal.info(self.description))
        l.append('')
        l.append(terminal.bold(_('Actors:').upper()))
        for i, actor in enumerate(self.actors):
            l.append('{}. {}'.format(i + 1, actor.name))
        l.append('')
        l.append(terminal.bold(_('Workflow:').upper()))
        for i, task in enumerate(self.workflow.tasks):
            l.append('{} {}'.format(' ' * i, task))
        l.append('')
        l.append(terminal.bold(_('Usecases:').upper()))
        for i, usecase in enumerate(self.usecases):
            l.append(terminal.bold('* Usecase #{}'.format((i + 1))))
            l.append('{}'.format(usecase))
        l.append('')
        l.append(terminal.bold(_('Class Diagrams:').upper()))
        for class_diagram in self.class_diagrams:
            l.append(terminal.bold('\t{} {}'.format(class_diagram.name, _('Diagram'))))
            for i, cls in enumerate(class_diagram.classes):
                l.append('\t\t\t{}. {}'.format(i + 1, cls['name']))
        return '\n'.join(l)




# -*- coding: utf-8 -*-

# Generated by Django 1.11 on 2017-09-10 15:43


from django.db import migrations
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0012_auto_20170619_1648'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='settings',
            name='sidebar_fixed',
        ),
        migrations.RemoveField(
            model_name='settings',
            name='sidebar_hidden',
        ),
        migrations.RemoveField(
            model_name='settings',
            name='sidebar_mini',
        ),
        migrations.RemoveField(
            model_name='settings',
            name='theme',
        ),
        migrations.AlterField(
            model_name='organization',
            name='ascii',
            field=djangoplus.db.models.fields.SearchField(blank=True, default=b'', editable=False, null=True),
        ),
        migrations.AlterField(
            model_name='settings',
            name='version',
            field=djangoplus.db.models.fields.CharField(max_length=255, null=True, verbose_name='Vers\xe3o do Sistema'),
        ),
        migrations.AlterField(
            model_name='unit',
            name='ascii',
            field=djangoplus.db.models.fields.SearchField(blank=True, default=b'', editable=False, null=True),
        ),
        migrations.AlterField(
            model_name='user',
            name='permission_mapping',
            field=djangoplus.db.models.fields.TextField(default=b'{}', verbose_name='Mapeamento de Permiss\xe3o'),
        ),
    ]



# -*- coding: utf-8 -*-

from . import views
from django.conf.urls import url


urlpatterns = [

    url(r'^list/(?P<app>\w+)/(?P<cls>\w+)/$', views.listt),
    url(r'^list/(?P<app>\w+)/(?P<cls>\w+)/(?P<subset>\w+)/$', views.listt),

    url(r'^add/(?P<app>\w+)/(?P<cls>\w+)/$', views.add),
    url(r'^add/(?P<app>\w+)/(?P<cls>\w+)/(?P<pk>\d+)/$', views.add),
    url(r'^add/(?P<app>\w+)/(?P<cls>\w+)/(?P<pk>\d+)/(?P<related_field_name>\w+)/$', views.add),
    url(r'^add/(?P<app>\w+)/(?P<cls>\w+)/(?P<pk>\d+)/(?P<related_field_name>\w+)/(?P<related_pk>\d+)/$', views.add),

    url(r'^view/(?P<app>\w+)/(?P<cls>\w+)/(?P<pk>\d+)/$', views.view),
    url(r'^view/(?P<app>\w+)/(?P<cls>\w+)/(?P<pk>\d+)/(?P<tab>[-\w]+)/$', views.view),

    url(r'^action/(?P<app>\w+)/(?P<cls>\w+)/(?P<action_name>\w+)/(?P<pk>\d+)/$', views.action),
    url(r'^action/(?P<app>\w+)/(?P<cls>\w+)/(?P<action_name>\w+)/$', views.action),

    url(r'^delete/(?P<app>\w+)/(?P<cls>\w+)/(?P<pk>\d+)/$', views.delete),
    url(r'^delete/(?P<app>\w+)/(?P<cls>\w+)/(?P<pk>\d+)/(?P<related_field_name>\w+)/(?P<related_pk>\d+)/$', views.delete),

    url(r'^log/(?P<app>\w+)/(?P<cls>\w+)/(?P<pk>\d+)/$', views.log),
    url(r'^log/(?P<app>\w+)/(?P<cls>\w+)/$', views.log),

    url(r'^(?P<app>\w+)/(?P<view_name>\w+)/(?P<params>.*)$', views.dispatcher),

]



# -*- coding: utf-8 -*-

# Generated by Django 1.10.5 on 2017-04-20 12:38


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0008_auto_20161124_1825'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='settings',
            name='activated',
        ),
        migrations.RemoveField(
            model_name='settings',
            name='admin_activation',
        ),
        migrations.RemoveField(
            model_name='settings',
            name='associated_groups',
        ),
        migrations.RemoveField(
            model_name='settings',
            name='mail_confirmation',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-03-09 16:13


from django.db import migrations
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0014_auto_20180106_1921'),
    ]

    operations = [
        migrations.AddField(
            model_name='settings',
            name='company',
            field=djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Empresa'),
        ),
    ]



# -*- coding: utf-8 -*-

import re
import json
import qrcode
import datetime
from decimal import Decimal
from django import template
from django.utils.safestring import mark_safe
from django.template.loader import render_to_string
from djangoplus.utils import permissions, http
from djangoplus.utils.formatter import format_value, normalyze
from djangoplus.utils.metadata import get_metadata, find_field_by_name, getattr2, is_many_to_many
from uuid import uuid4
from django.template.defaultfilters import slugify
import base64
import tempfile
register = template.Library()


@register.filter
def tojson(value):
    return json.dumps(value)


@register.filter
def short_username(value):
    return value.split('@')[0]


@register.filter
def displayable(value):
    return value not in [True, False, None, '']


@register.filter
def get_item(dictionary, key):
    return dictionary.get(key)


@register.simple_tag(name='uuid')
def do_uuid4():
    return uuid4()


@register.simple_tag()
def value_at(col_value, col_index, template_filters):
    from . import utils
    value = col_value

    # Se há um template filter com o mesmo índice da coluna, então
    # é sinal que este deverá ser o template_filter a ser utilizado.
    if col_index in template_filters:
        template_filter = template_filters[col_index]
        value = utils.apply_filter(value, template_filter)

    # Realizando a formatação padrão.
    value = utils.apply_filter(value, 'format2')
    return value


@register.simple_tag()
def tree_info(obj, queryset):
    if hasattr(obj, 'get_parent_field'):
        parent_field = obj.get_parent_field()
        if parent_field:
            parent = getattr(obj, parent_field.name)
            if not hasattr(queryset, '__pks'):
                queryset.__pks = queryset.values_list('pk', flat=True)
            if parent and parent.pk in queryset.__pks:
                return 'treegrid-{}  treegrid-parent-{}'.format(obj.pk, parent.pk)
            else:
                return 'treegrid-{}'.format(obj.pk)
    else:
        return ''


@register.simple_tag()
def paginator_icons(paginator, obj):
    edit = not paginator.readonly
    delete = not paginator.readonly
    return obj_icons(paginator.request, obj, to=paginator.to, edit=edit, delete=delete)


@register.simple_tag()
def obj_icons(request, obj, to=None, edit=True, delete=True, css='ajax'):
    l = []
    model = obj.__class__
    cls = model.__name__.lower()
    app = get_metadata(model, 'app_label')
    css = to and 'popup' or css
    parent = to and hasattr(obj, to) and getattr(obj, to) or None
    if hasattr(obj, 'get_absolute_url'):
        view_url = obj.get_absolute_url()
    else:
        view_url = '/view/{}/{}/{}/'.format(app, cls, obj.pk)
    url_id = slugify(view_url)
    url_title = 'Visualizar'

    if parent:
        from djangoplus.cache import loader
        can_spy = True
        if type(obj) in loader.actions:
            for category in loader.actions[type(obj)]:
                for action_name in loader.actions[type(obj)][category]:
                    action = loader.actions[type(obj)][category][action_name]
                    if not action['inline']:
                        can_spy = False
                        break
        fieldsets = get_metadata(obj, 'fieldsets')
        if fieldsets:
            for fieldset in fieldsets:
                if '::' in fieldset[0]:
                    can_spy = False
                    break

        l.append('<a id="{}" class="{}" href="{}?parent={}" title="{}"><i class="fa fa-search fa-lg"></i></a>'.format(url_id, 'ajax', view_url, to, url_title))
        if can_spy:
            l.append('<a id="{}" class="{}" href="{}?parent={}" title="{}"><i class="fa fa-eye fa-lg"></i></a>'.format(url_id, 'popup', view_url, to, url_title))
    else:
        l.append('<a id="{}" class="{}" href="{}" title="{}"><i class="fa fa-search fa-lg"></i></a>'.format(url_id, to and 'popup' or 'ajax', view_url, url_title))

    if edit or delete:
        if parent and is_many_to_many(parent.__class__, to):
            if edit and (not hasattr(obj, 'can_edit') or obj.can_edit()):
                delete_url = '/delete/{}/{}/{}/{}/{}/'.format(app, parent.__class__.__name__.lower(), parent.pk, to, obj.pk)
                url_id = slugify(delete_url)
                url_title = 'Remover'
                l.append(' <a id="{}" class="{}" href="{}" title="{}"><i class="fa fa-times fa-lg"></i></a>'.format(url_id, 'ajax', delete_url, url_title))
        else:
            if edit and permissions.has_edit_permission(request, type(obj)) and (not hasattr(obj, 'can_edit') or obj.can_edit()):
                if parent:
                    _app = get_metadata(parent.__class__, 'app_label')
                    _cls = parent.__class__.__name__.lower()
                    _related_field = find_field_by_name(obj.__class__, to).remote_field.get_accessor_name()
                    edit_url = '/add/{}/{}/{}/{}/{}/'.format(_app, _cls, parent.pk, _related_field, obj.pk)
                else:
                    edit_url = '/add/{}/{}/{}/'.format(app, cls, obj.pk)
                url_id = slugify(edit_url)
                url_title = 'Editar'
                l.append(' <a id="{}" class="{}" href="{}" title="{}"><i class="fa fa-edit fa-lg"></i></a>'.format(url_id, css, edit_url, url_title))

            if delete and permissions.has_delete_permission(request, type(obj)) and (not hasattr(obj, 'can_delete') or obj.can_delete()):
                delete_url = '/delete/{}/{}/{}/'.format(app, cls, obj.pk)
                url_id = slugify(delete_url)
                url_title = 'Excluir'
                l.append(' <a id="{}" class="{}" href="{}" title="{}"><i class="fa fa-trash-o fa-lg"></i></a>'.format(url_id, 'popup', delete_url, url_title))

            tree_index_field = hasattr(obj, 'get_tree_index_field') and obj.get_tree_index_field() or None
            if tree_index_field:
                add_url = '/add/{}/{}/{}/{}/'.format(app, cls, obj.pk, cls)
                url_id = slugify(view_url)
                url_title = 'Adicionar'
                l.append(' <a id="{}" class="{}" href="{}" title="{}"><i class="fa fa-plus fa-lg"></i></a>'.format(
                url_id, 'popup', add_url, url_title))

    return mark_safe(''.join(l))


@register.filter
def red_if_negative(obj):
    value = format2(obj)
    if obj < 0:
        return '<span class="text-danger">{}</span>'.format(value)
    return value


@register.filter
def children(obj):
    return getattr(obj, '{}_set'.format(obj.__class__.__name__.lower())).all()


@register.filter
def link(value):
    if value:
        app = get_metadata(value.__class__, 'app_label')
        cls = value.__class__.__name__.lower()

        return mark_safe('<a class="ajax" href="/view/{}/{}/{}/">{}</a>'.format(app, cls, value.pk, value))
    return '-'


@register.filter(name='format')
def format2(value, request=None):
    if value and hasattr(value, 'id') and hasattr(value, '_meta') and hasattr(value, 'fieldsets'):
        app = get_metadata(value.__class__, 'app_label')
        cls = value.__class__.__name__.lower()
        if request and request.user.has_perm('{}.{}'.format(app, cls)):
            return link(value)
    return mark_safe(format_value(value).replace('\n', '<br />'))


@register.filter
def print_format(value):
    return mark_safe(format_value(value).replace('\n', '<br />'))


@register.filter
def ordered_list(value):
    l = ['<ol style="padding-left:13px">']
    for obj in value:
        l.append('<li style="list-style-type:decimal">{}</li>'.format(obj))
    l.append('</ol>')
    return mark_safe(''.join(l))


@register.filter()
def mobile(request):
    return http.mobile(request)


@register.filter()
def align(value):
    position ='left'
    if isinstance(value, bool) or isinstance(value, datetime.date):
        position='center'
    elif isinstance(value, Decimal) or isinstance(value, int):
        position='right'
    elif isinstance(value, tuple):
        return align(value[0])
    return position


@register.filter
def render_paginator(paginator):
    list_template = get_metadata(paginator.qs.model, 'list_template')
    return render_to_string(list_template, {'paginator' : paginator})


@register.filter
def add_actions(paginator, obj):
    paginator.drop_down.add_actions(obj, inline=True, subset_name=paginator.get_current_tab_name() or None)
    return ''


@register.filter
def html(value):
    if value:
        value = str(value)
        value = value.replace('\n', '<br>').replace('  ', '&nbsp; ').replace('\t', '&nbsp;&nbsp;&nbsp;&nbsp;')
        return mark_safe(value)
    else:
        return '-'


@register.filter
def number(value):
    return str(value)


@register.filter
def tahoma(value):
    return mark_safe('<font style="font-family: tahoma">{}</font>'.format(html(value)))


@register.filter
def colspan(n):
    return int(12/n)


@register.filter
def must_hide_fieldset(tuples):
    count = 0
    for fields in tuples:
        count += len(fields)
    return count == 0


register.filter('getattr', getattr2)
register.filter('normalyze', normalyze)


@register.filter
def sorted_items(d):
    items = []
    keys = list(d.keys())
    keys.sort()
    for key in keys:
        items.append((re.sub('^[0-9\.\- ]+', '', key).strip(), d[key]))
    return items


@register.filter
def photo(user):
    return user and user.photo and '/media/{}'.format(user.photo) or '/static/images/user.png'


@register.filter
def toast(message):
    return mark_safe("<script>$.toast({{ text: '{}', loader: false, position : {{top: 60, right: 30}}, hideAfter: 10000}});</script>".format(message.message))


@register.simple_tag()
def set_request(obj, request):
    obj.request = request
    return ''


@register.filter
def is_image(value):
    if value:
        for ext in ('.png', '.jpg', '.jpeg', '.gif'):
            if str(value).lower().endswith(ext):
                return True
    return False


@register.filter
def qrcode64(text):
    qr = qrcode.QRCode()
    qr.add_data(text)
    image = qr.make_image()
    file_path = tempfile.mktemp()
    buffer = open(file_path, 'wb')
    image.save(buffer, format="JPEG")
    img_str = base64.b64encode(open(file_path, 'rb').read()).decode('utf-8')
    return mark_safe('<img width="100" src="data:image/jpeg;base64, {}"/>'.format(img_str))


# -*- coding: utf-8 -*-

import json
import six
import sys
from django.core.exceptions import ValidationError
from django.db.models import base
from django.db.models import Q
from django.conf import settings
from django.db.models import query
from operator import __or__ as OR
from djangoplus.db.models.fields import *
import django.db.models.options as options
from django.db.models.aggregates import Sum, Avg
from django.db.models.deletion import Collector
from djangoplus.utils.metadata import get_metadata, getattr2, find_model, get_field
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth import get_user_model
from functools import reduce

setattr(options, 'DEFAULT_NAMES', options.DEFAULT_NAMES + (
    'icon', 'verbose_female', 'order_by', 'pdf', 'menu',
    'list_display', 'list_per_page', 'list_template', 'list_total', 'list_shortcut', 'list_csv', 'list_xls', 'list_menu', 'list_lookups', 'list_pdf',
    'add_label', 'add_form', 'add_inline', 'add_message', 'add_shortcut', 'select_template', 'select_display', 'select_related',
    'role_name', 'role_username', 'role_email', 'role_scope', 'role_signup', 'role_active',
    'log', 'logging',
    'can_add', 'can_edit', 'can_delete', 'can_list', 'can_view', 'can_admin',
    'can_list_by_role', 'can_view_by_role', 'can_add_by_role', 'can_edit_by_role', 'can_admin_by_role',
    'can_list_by_unit', 'can_view_by_unit', 'can_add_by_unit', 'can_edit_by_unit', 'can_admin_by_unit',
    'can_list_by_organization', 'can_view_by_organization', 'can_add_by_organization', 'can_edit_by_organization', 'can_admin_by_organization',
    'usecase', 'class_diagram',
))


class QueryStatistics(object):
    def __init__(self, series, labels, groups=list(), title=None):
        self.title = title
        self.labels = labels
        self.groups = groups
        self.series = []
        self.xtotal = []
        self.ytotal = []

        for serie in series:
            self.add(serie)

    def add(self, serie, avg=False):
        self.series.append(serie)
        if avg:
            self.xtotal.append(sum(serie)/len(serie))
        else:
            self.xtotal.append(sum(serie))
        if not self.ytotal:
            for value in serie:
                self.ytotal.append(value)
        else:
            for i, value in enumerate(serie):
                self.ytotal[i] += value

    def total(self):
        return sum(self.xtotal)

    def __str__(self):
        return '{}\n{}\n{}'.format(self.labels, self.series, self.groups)

    def as_table(self, title=None, symbol=None):
        from djangoplus.ui.components.utils import StatisticsTable
        return StatisticsTable(None, self.title, self, symbol=symbol)

    def as_chart(self, title=None, symbol=None):
        return self.as_table(title=title, symbol=symbol).as_chart()


class ModelGeneratorWrapper:
    def __init__(self, generator, user):
        self.generator = generator
        self._user = user

    def __iter__(self):
        return self

    def __next__(self):
        obj = next(self.generator)
        obj._user = self._user
        return obj


class ModelIterable(query.ModelIterable):

    def __iter__(self):
        generator = super(ModelIterable, self).__iter__()
        return ModelGeneratorWrapper(generator, self.queryset._user)


class QuerySet(query.QuerySet):

    def __init__(self, *args, **kwargs):
        self._user = None
        super(QuerySet, self).__init__(*args, **kwargs)
        self._iterable_class = ModelIterable

    def _clone(self):
        clone = super(QuerySet, self)._clone()
        clone._user = self._user
        return clone

    def all(self, user=None, obj=None):
        app_label = get_metadata(self.model, 'app_label')
        if user:
            role_username = get_metadata(user, 'role_username')
            if role_username:
                user = get_user_model().objects.get(username=getattr(user, role_username))
        queryset = self._clone()
        queryset._user = user
        if user:
            self_permission = '{}.view_{}'.format(app_label, self.model.__name__.lower())
            obj_permission = obj and '{}.view_{}'.format(get_metadata(type(obj), 'app_label'), type(obj).__name__.lower())
            has_perm = obj_permission and user.has_perm(obj_permission) or user.has_perm(self_permission)
        else:
            has_perm = True
        if has_perm:
            if user and (user.organization_id or user.unit_id or not user.is_superuser):
                permission_mapping = user.get_permission_mapping(self.model, obj)
                if 'list_lookups' in permission_mapping and permission_mapping['list_lookups']:
                    l = []
                    for lookup, value in permission_mapping['list_lookups']:
                        l.append(Q(**{'{}__in'.format(lookup): value}))
                    return queryset.filter(reduce(OR, l))
            return queryset
        return self.none()
     
    def _calculate(self, vertical_key=None, horizontal_key=None, aggregate=None):
        verbose_name = get_metadata(self.model,  'verbose_name')
        if not vertical_key:
            if aggregate:
                value = 0
                mode, attr = aggregate
                if mode == 'sum':
                    value = self.aggregate(Sum(attr)).get('{}__sum'.format(attr)) or 0
                elif mode == 'avg':
                    value = self.aggregate(Avg(attr)).get('{}__avg'.format(attr)) or 0
                aggregation_field = get_field(self.model, attr)
                if type(aggregation_field).__name__ in ('DecimalField',):
                    value = Decimal(value)
                return value
            return None
        vertical_field = get_field(self.model, vertical_key)
        if type(vertical_field).__name__ in ('DateField', 'DateTimeField'):
            months = ['Jan', 'Fev', 'Mar', 'Abr', 'Mai', 'Jun', 'Jul', 'Ago', 'Set', 'Out', 'Nov', 'Dez']

            if horizontal_key:
                iterator_model = get_field(self.model, horizontal_key).remote_field.model
                iterators = iterator_model.objects.filter(pk__in=self.values_list(horizontal_key, flat=True).order_by(horizontal_key).distinct())
                horizontal_field = get_field(self.model, horizontal_key)
                title = '{} anual por {}'.format(verbose_name, horizontal_field.verbose_name)
                statistics = QueryStatistics([], [str(x) for x in iterators], months, title=title)

                for iterator in iterators:
                    serie = []
                    for i, month in enumerate(months):
                        qs = self.filter(**{'{}__month'.format(vertical_key): i+1, horizontal_key: iterator.pk})
                        serie.append(qs.count())
                    statistics.add(serie)
                return statistics
            else:
                title = '{} Anual'.format(verbose_name)
                statistics = QueryStatistics([], months, title=title)
                serie = []
                for i, month in enumerate(months):
                    if aggregate:
                        total = 0
                        mode, attr = aggregate
                        if mode == 'sum':
                            total = self.filter(**{'{}__month'.format(vertical_key): i+1}).aggregate(Sum(attr)).get('{}__sum'.format(attr)) or 0
                        elif mode == 'avg':
                            total = self.filter(**{'{}__month'.format(vertical_key): i+1}).aggregate(Avg(attr)).get('{}__avg'.format(attr)) or 0
                        aggregation_field = get_field(self.model, attr)
                        if type(aggregation_field).__name__ in ('DecimalField',):
                            total = Decimal(total)
                    else:
                        total = self.filter(**{'{}__month'.format(vertical_key): i+1}).count()
                    serie.append(total)
                statistics.add(serie)
                return statistics
        else:
            if vertical_field.choices:
                vertical_choices = vertical_field.choices
            elif vertical_field.__class__.__name__ == 'BooleanField':
                vertical_choices = [(True, 'Sim'), (False, 'Não')]
            else:
                vertical_model = find_model(self.model, vertical_key)
                vertical_choices = [(o.pk, str(o)) for o in vertical_model.objects.filter(id__in=self.values_list(vertical_key, flat=True))]
            if horizontal_key:
                horizontal_field = get_field(self.model, horizontal_key)
                if horizontal_field.choices:
                    horizontal_choices = horizontal_field.choices
                elif horizontal_field.__class__.__name__ == 'BooleanField':
                    vertical_choices = [(True, 'Sim'), (False, 'Não')]
                else:
                    horizontal_model = find_model(self.model, horizontal_key)
                    horizontal_choices = [(o.pk, str(o)) for o in horizontal_model.objects.filter(id__in=self.values_list(horizontal_key, flat=True))]

                title = '{} por {} e {}'.format(verbose_name, vertical_field.verbose_name.lower(), horizontal_field.verbose_name)
                statistics = QueryStatistics([], [choice[1] for choice in vertical_choices], [choice[1] for choice in horizontal_choices], title=title)
                for vertical_choice in vertical_choices:
                    serie = []
                    avg = False
                    for horizontal_choice in horizontal_choices:
                        value = 0
                        lookup = {vertical_key: vertical_choice[0], horizontal_key: horizontal_choice[0]}
                        if aggregate:
                            mode, attr = aggregate
                            if mode == 'sum':
                                value = self.filter(**lookup).aggregate(Sum(attr)).get('{}__sum'.format(attr)) or 0
                            elif mode == 'avg':
                                avg = True
                                value = self.filter(**lookup).aggregate(Avg(attr)).get('{}__avg'.format(attr)) or 0
                            aggregation_field = get_field(self.model, attr)
                            if type(aggregation_field).__name__ in ('DecimalField',):
                                value = Decimal(value)
                        else:
                            value = self.filter(**lookup).values('id').count()
                        serie.append(value)
                    statistics.add(serie, avg)
                return statistics
            else:
                title = '{} por {}'.format(verbose_name, vertical_field.verbose_name)
                statistics = QueryStatistics([], [choice[1] for choice in vertical_choices], title=title)
                serie = []
                avg = False
                for vertical_choice in vertical_choices:
                    lookup = {vertical_key: vertical_choice[0]}
                    value = 0
                    if aggregate:
                        mode, attr = aggregate
                        if mode == 'sum':
                            value = self.filter(**lookup).aggregate(Sum(attr)).get('{}__sum'.format(attr)) or 0
                        elif mode == 'avg':
                            avg = True
                            value = self.filter(**lookup).aggregate(Avg(attr)).get('{}__avg'.format(attr)) or 0
                        aggregation_field = get_field(self.model, attr)
                        if type(aggregation_field).__name__ in ('DecimalField',):
                            value = Decimal(value)
                    else:
                        value = self.filter(**lookup).count()
                    serie.append(value)
                statistics.add(serie, avg)
                return statistics

    def count(self, vertical_key=None, horizontal_key=None):
        if vertical_key:
            return self._calculate(vertical_key, horizontal_key, aggregate=None)
        else:
            return super(QuerySet, self).count()

    def sum(self, attr, vertical_key=None, horizontal_key=None):
        return self._calculate(vertical_key, horizontal_key, aggregate=('sum', attr))

    def avg(self, attr, vertical_key=None, horizontal_key=None):
        return self._calculate(vertical_key, horizontal_key, aggregate=('avg', attr))
    
    @classmethod
    def as_manager(cls):
        return Manager(queryset_class=cls)


class Manager(models.Manager):

    def __init__(self, *args, **kwargs):
        self.queryset_class = kwargs.pop('queryset_class', QuerySet)
        self.request = None
        super(Manager, self).__init__(*args, **kwargs)

    def get_queryset(self):
        return self.queryset_class(self.model, using=self._db)

    def all(self, user=None, obj=None):
        return self.get_queryset().all(user, obj=obj)

    def count(self, vertical_key=None, horizontal_key=None):
        return self.get_queryset().count(vertical_key, horizontal_key)

    def sum(self, attr, vertical_key=None, horizontal_key=None):
        return self.get_queryset().sum(attr, vertical_key, horizontal_key)


class ModelBase(base.ModelBase):
    def __new__(mcs, name, bases, attrs):
        meta_new = super(ModelBase, mcs).__new__
        module = __import__(attrs['__module__'], fromlist=list(map(str, attrs['__module__'].split('.'))))
        if hasattr(module, '{}Manager'.format(name)):
            queryset_class = getattr(module, '{}Manager'.format(name))
            if issubclass(queryset_class, QuerySet):
                attrs.update(objects=Manager(queryset_class=queryset_class))

        cls = meta_new(mcs, name, bases, attrs)
        for attr_name in attrs:
            if attr_name not in ('objects',):
                attr = getattr(cls, attr_name)
                if hasattr(attr, '_action'):
                    pass

        requires_tree_index = False
        declared_tree_index = False
        for field in cls._meta.fields:
            if field.__class__.__name__ == 'TreeIndexField':
                declared_tree_index = True
            if field.__class__.__name__ == 'ForeignKey' and field.tree:
                requires_tree_index = True

        if requires_tree_index:
            if not hasattr(cls._meta, 'select_template'):
                setattr(cls._meta, 'select_template', 'tree_node_option.html')
            if not declared_tree_index:
                from .fields import TreeIndexField
                cls.add_to_class('tree_index', TreeIndexField())

        return cls

DefaultManager = QuerySet


class Model(six.with_metaclass(ModelBase, models.Model)):

    class Meta:
        abstract = True

    objects = DefaultManager.as_manager()

    def __init__(self, *args, **kwargs):
        super(Model, self).__init__(*args, **kwargs)
        self.request = None
        self._user = None

    def __str__(self):
        for subclass in self.__class__.__subclasses__():
            if hasattr(self, subclass.__name__.lower()):
                subinstance = getattr(self, subclass.__name__.lower())
                if hasattr(subinstance, '__str__'):
                    return subinstance.__str__()
        return '{}{}'.format(get_metadata(self.__class__, 'verbose_name'), self.pk and ' #{}'.format(self.pk or ''))

    @classmethod
    def update_metadata(cls, **kwargs):
        for attr in kwargs:
            value = kwargs[attr]
            setattr(getattr(cls, '_meta'), attr, value)

    @classmethod
    def is_tree(cls):
        return cls.get_tree_index_field()

    @classmethod
    def get_tree_index_field(cls):
        for field in cls._meta.fields:
            if type(field).__name__ == 'TreeIndexField':
                return field
        return None

    @classmethod
    def get_parent_field(cls):
        for field in cls._meta.fields:
            if field.remote_field and field.remote_field.model == cls:
                return field
        return None

    def get_depth(self):
        tree_index_field = self.get_tree_index_field()
        if tree_index_field:
            return len((getattr(self, tree_index_field.name) or '').split(tree_index_field.sep))-1
        return 0

    def get_children(self):
        tree_index = getattr(self, self.get_tree_index_field().name)
        return type(self).objects.filter(**{'{}__startswith'.format(tree_index):tree_index}).exclude(pk=self.pk)

    def save(self, *args, **kwargs):
        log_data = get_metadata(self.__class__, 'log', False)
        log_index = get_metadata(self.__class__, 'logging', (), iterable=True)

        # edited changes logging
        if (log_data or log_index) and self.pk and self._user:
            log_indexes = list(log_index)
            qs = self.__class__.objects.filter(pk=self.pk).select_related(*log_indexes)

            self._diff = False
            if qs.exists():
                from djangoplus.admin.models import Log
                old = qs[0]
                log = Log()
                log.operation = Log.EDIT
                log.user = self._user
                log.content_type = ContentType.objects.get_for_model(self.__class__)
                log.object_id = self.pk
                log.object_description = str(self)
                diff = []

                for field in get_metadata(self, 'fields'):
                    o1 = getattr(old, field.name)
                    o2 = getattr(self, field.name)
                    v1 = str(o1)
                    v2 = str(o2)
                    if not hasattr(o2, '__iter__') and v1 != v2:
                        self._diff = True
                        diff.append((field.verbose_name, v1, v2))

                log.content = json.dumps(diff)
                log.save()
                log.create_indexes(old)

        # tree model
        tree_index_field = self.get_tree_index_field()
        if tree_index_field:
            recursive = kwargs.pop('recursive', False)
            parent_field = self.get_parent_field()
            parent = getattr(self, parent_field.name)
            if tree_index_field.ref:
                tree_index_end = getattr(self, tree_index_field.ref)
                if not tree_index_end:
                    raise Exception('{} deve ser informado.'.format(tree_index_field.base))
            else:
                tree_index_end = self.pk and getattr(self, tree_index_field.name).split('.')[-1] or None
                if recursive or not self.pk or not type(self).objects.filter(
                        **{'id': self.pk, parent_field.name: parent}).exists():
                    tree_index_end = '1'
                    qs = type(self).objects.filter(**{parent_field.name: parent}).order_by(
                        '-{}'.format(tree_index_field.name)).values_list(tree_index_field.name, flat=True)
                    if qs.exists():
                        tree_index_end = int(qs[0].split('.')[-1]) + 1
            if parent:
                parent_index = getattr(parent, tree_index_field.name)
                tree_index = '{}{}{}'.format(parent_index, tree_index_field.sep, tree_index_end)
            else:
                tree_index = tree_index_end

            setattr(self, tree_index_field.name, tree_index)

        super(Model, self).save(*args, **kwargs)

        # make many-to-many and one-to-many relations available after save
        if hasattr(self, '_post_save_form'):
            post_save_form = getattr(self, '_post_save_form')
            getattr(post_save_form, 'save_121_and_12m')()
            getattr(post_save_form, '_save_m2m')()
            super(Model, self).save(*args, **kwargs)

        if tree_index_field:
            for obj in type(self).objects.filter(**{parent_field.name: self}):
                kwargs['recursive'] = True
                obj.save(*args, **kwargs)

        # registration logging
        if (log_data or log_index) and not hasattr(self, '_diff') and self._user:
            from djangoplus.admin.models import Log
            log = Log()
            log.operation = Log.ADD
            log.user = self._user
            log.content_type = ContentType.objects.get_for_model(self.__class__)
            log.object_id = self.pk
            log.object_description = str(self)
            log.content = '[]'
            log.save()
            log.create_indexes(self)

        self._check_role()

    def as_user(self):
        role_username = get_metadata(self.__class__, 'role_username')
        if role_username:
            return get_user_model().objects.get(username=getattr(self, role_username))
        return None

    def _check_role(self, saving=True):
        role_name = get_metadata(self.__class__, 'role_name')
        role_username = get_metadata(self.__class__, 'role_username')
        verbose_name = get_metadata(self.__class__, 'verbose_name')
        concrete_fields = get_metadata(self.__class__, 'concrete_fields')
        role_email = get_metadata(self.__class__, 'role_email', '')
        role_scope = get_metadata(self.__class__, 'role_scope')

        if role_username:

            from django.contrib.auth.models import Group
            from djangoplus.admin.models import Role, Organization, OrganizationRole, Unit, UnitRole

            group_name = verbose_name
            username = getattr2(self, role_username)
            name = role_name and getattr2(self, role_name) or None

            if username:
                unit = Unit.objects.get(pk=0)
                organization = organization = Organization.objects.get(pk=0)

                if issubclass(self.__class__, Organization):
                    organization = self
                    unit = None
                elif issubclass(self.__class__, Unit):
                    unit = self
                    organization = None
                elif role_scope:
                    scope = getattr2(self, role_scope)
                    if issubclass(scope.__class__, Organization):
                        organization = scope
                        unit = None
                    elif issubclass(scope.__class__, Unit):
                        unit = scope
                        organization = None
                else:
                    for field in concrete_fields:
                        if field.remote_field and field.remote_field.model:
                            if issubclass(field.remote_field.model, Organization):
                                organization = getattr(self, field.name)
                                unit = None
                                break
                            if issubclass(field.remote_field.model, Unit):
                                unit = getattr(self, field.name)
                                organization = None
                                break

                email = role_email and getattr2(self, role_email) or ''

                User = get_user_model()
                group = Group.objects.get_or_create(name=group_name)[0]

                if unit and not organization:
                    for field in get_metadata(unit.__class__, 'concrete_fields'):
                        if field.remote_field and field.remote_field.model:
                            if issubclass(field.remote_field.model, Organization):
                                organization = getattr(unit, field.name)

                if saving:
                    qs = User.objects.filter(username=username)
                    if qs.exists():
                        user = qs[0]
                        user.email = email
                        user.name = name or str(self)
                        user.save()
                    else:
                        user = User()
                        user.username = username
                        user.name = name or str(self)
                        user.email = email
                        user.save()
                        if user.email and not (settings.DEBUG or 'test' in sys.argv):
                            user.send_access_invitation()

                    user.groups.add(group)
                    if organization:
                        role = Role.objects.get(user=user, group=group)
                        if not OrganizationRole.objects.filter(role=role, organization=organization).exists():
                            organization_role = OrganizationRole()
                            organization_role.role = role
                            organization_role.organization = organization
                            organization_role.save()
                    if unit:
                        role = Role.objects.get(user=user, group=group)
                        if not UnitRole.objects.filter(role=role, unit=unit).exists():
                            unit_role = UnitRole()
                            unit_role.role = role
                            unit_role.unit = unit
                            unit_role.save()
                else:
                    user = User.objects.get(username=username)
                    if unit or organization:
                        keep_in_group = False
                        if organization:
                            OrganizationRole.objects.filter(role__user=user, role__group=group, organization=organization).delete()
                            if OrganizationRole.objects.filter(role__user=user, role__group=group).exists():
                                keep_in_group=True
                        if unit:
                            UnitRole.objects.filter(role__user=user, role__group=group, unit=unit).delete()
                            if UnitRole.objects.filter(role__user=user, role__group=group).exists():
                                keep_in_group = True
                        if not keep_in_group:
                            user.groups.remove(group)
                    else:
                        UnitRole.objects.filter(role__user=user, role__group=group).delete()
                        user.groups.remove(group)

    def delete(self, *args, **kwargs):
        log_data = get_metadata(self.__class__, 'log', False)
        log_index = get_metadata(self.__class__, 'logging', ())

        if (log_data or log_index) and self._user:
            from djangoplus.admin.models import Log

            collector = Collector(using='default')
            collector.collect([self], keep_parents=False)
            for cls, objs in list(collector.data.items()):
                content_type = ContentType.objects.get_for_model(cls)
                for obj in objs:
                    log = Log()
                    log.operation = Log.DELETE
                    log.user = self._user
                    log.content_type = content_type
                    log.object_id = obj.pk
                    log.object_description = str(obj)
                    diff = []
                    for field in get_metadata(obj.__class__, 'fields'):
                        if not isinstance(field, models.FileField):
                            o1 = getattr(obj, field.name)
                            v1 = str(o1)
                            diff.append((field.verbose_name, v1))

                    log.content = json.dumps(diff)
                    log.save()
                    log.create_indexes(obj)

        super(Model, self).delete(*args, **kwargs)

        self._check_role(False)

    def get_logs(self):
        content_type = ContentType.objects.get_for_model(self.__class__)
        from djangoplus.admin.models import Log
        qs1 = Log.objects.filter(content_type=content_type, object_id=self.pk)
        qs2 = Log.objects.filter(logindex__content_type=content_type, logindex__object_id=self.pk)
        return (qs1 | qs2).order_by('-date')

    def can_edit(self):
        if self._user:
            if self._user.organization_id or self._user.unit_id or not self._user.is_superuser:
                model = type(self)
                app_label = get_metadata(model, 'app_label')
                perm_name = '{}.edit_{}'.format(app_label, model.__name__.lower())
                if self._user.has_perm(perm_name):
                    permission_mapping = self._user.get_permission_mapping(model)
                    if 'edit_lookups' in permission_mapping and permission_mapping['edit_lookups']:
                        for lookup, values in permission_mapping['edit_lookups']:
                            value = getattr2(self, lookup)
                            value = hasattr(value, 'pk') and value.pk or value
                            if value in values:
                                return True
                        return False
                else:
                    return False
        return True

    def can_delete(self):
        if self._user:
            if self._user.organization_id or self._user.unit_id or not self._user.is_superuser:
                model = type(self)
                app_label = get_metadata(model, 'app_label')
                perm_name = '{}.delete_{}'.format(app_label, model.__name__.lower())
                if self._user.has_perm(perm_name):
                    permission_mapping = self._user.get_permission_mapping(model)
                    if 'delete_lookups' in permission_mapping and permission_mapping['delete_lookups']:
                        for lookup, values in permission_mapping['delete_lookups']:
                            value = getattr2(self, lookup)
                            value = hasattr(value, 'pk') and value.pk or value
                            if value in values:
                                return True
                        return False
                else:
                    return False
        return True


class AsciiModel(Model):
    class Meta:
        abstract = True

    ascii = SearchField(blank=True, default='', editable=False, search=True, exclude=True)




# -*- coding: utf-8 -*-

from djangoplus.ui.components.utils import Timeline, StatisticsTable, QrCode, ProgressBar


def timeline(value, **kwargs):
    return Timeline(kwargs.get('request'),  kwargs.get('verbose_name'), value)


def statistics(value, **kwargs):
    return StatisticsTable(kwargs.get('request'), kwargs.get('verbose_name'), value)


def chart(value, **kwargs):
    return StatisticsTable(kwargs.get('request'), kwargs.get('verbose_name'), value).as_chart()


def pie_chart(value, **kwargs):
    return chart(value, **kwargs).pie()


def donut_chart(value, **kwargs):
    return chart(value, **kwargs).donut()


def bar_chart(value, **kwargs):
    return chart(value, **kwargs).bar()


def horizontal_bar_chart(value, **kwargs):
    return chart(value, **kwargs).horizontal_bar()


def stack_chart(value, **kwargs):
    return chart(value, **kwargs).stack()


def horizontal_stack_chart(value, **kwargs):
    return chart(value, **kwargs).horizontal_stack()


def line_chart(value, **kwargs):
    return chart(value, **kwargs).line()


def area_chart(value, **kwargs):
    return chart(value, **kwargs).area()


def box_chart(value, **kwargs):
    return chart(value, **kwargs).box()


def qrcode(value, **kwargs):
    request = kwargs.get('request', None)
    return QrCode(request, value)


def progress(percentual):
    return ProgressBar(percentual)




# -*- coding: utf-8 -*-

from django.forms import widgets
from djangoplus.ui.components.calendar.widgets import *
from djangoplus.ui.components.editor.widgets import *
from djangoplus.ui.components.select.widgets import *

# Base Widgets #


class TextInput(widgets.TextInput):
    def render(self, name, value, attrs={}):
        attrs.update(**{'class':'form-control {}'.format(self.attrs.get('class', ''))})
        return super(TextInput, self).render(name, value, attrs)


class EmailInput(TextInput):
    def render(self, name, value, attrs={}):
        attrs['type'] = 'email'
        html = super(EmailInput, self).render(name, value, attrs)
        html = '''
        <div class="input-group">
            {}
            <span class="input-group-addon"><i class="fa fa-at"></i></span>
        </div>
        '''.format(html)
        return html


class UrlInput(TextInput):
    def render(self, name, value, attrs={}):
        attrs['type'] = 'url'
        html = super(UrlInput, self).render(name, value, attrs)
        html = '''
        <div class="input-group">
            {}
            <span class="input-group-addon"><i class="fa fa-link"></i></span>
        </div>
        '''.format(html)
        return html


class AddressInput(TextInput):
    def render(self, name, value, attrs={}):
        html = super(AddressInput, self).render(name, value, attrs)
        html = '''
        <div class="input-group">
            {}
            <span class="input-group-addon"><i class="fa fa-envelope-o"></i></span>
        </div>
        '''.format(html)
        return html


class MaskWidget(widgets.TextInput):

    class Media:
        js = ('/static/js/jquery.mask.min-1.7.7.js',)

    def __init__(self, mask):
        super(MaskWidget, self).__init__()
        self.mask = mask

    def render(self, name, value, attrs={}):
        attrs['class'] = 'form-control'
        html = super(MaskWidget, self).render(name, value, attrs)
        script = '<script>$("#id_{}").mask("{}", {{clearIfNotMatch: true}});</script>'.format(name, self.mask)
        return mark_safe('{}\n\n{}'.format(html, script))


class HiddenInput(widgets.HiddenInput):
    pass


class ReadOnlyInput(TextInput):
    def render(self, name, value, attrs=None):
        self.attrs['readonly'] = 'readonly'
        return super(ReadOnlyInput, self).render(name, value, attrs)


class DisplayInput(widgets.TextInput):
    def __init__(self, obj):
        super(DisplayInput, self).__init__()
        self.obj = obj

    def render(self, name, value, attrs=None):
        return '<input class="form-control" type="text" value="{}" disabled /><input type="hidden" id="id_{}" name="{}" value="{}"/>'.format(self.obj, name, name, self.obj.pk)


class Textarea(widgets.Textarea):
    def render(self, name, value, attrs={}):
        attrs['class'] = 'form-control'
        if 'rows' not in attrs: # and 'rows' not in self.attrs
            attrs['rows'] = 3
        html = super(Textarea, self).render(name, value, attrs)
        return mark_safe(html)



class NumberInput(widgets.NumberInput):
    def render(self, name, value, attrs={}):
        attrs['class'] = 'form-control'
        html = super(NumberInput, self).render(name, value, attrs)
        if self.max_value is not None and self.min_value is not None:
            html = html # TODO range widget

        return mark_safe(html)


class DecimalInput(widgets.TextInput):

    class Media:
        js = ('/static/js/jquery.mask.min-1.7.7.js',)

    def render(self, name, value, attrs={}):
        attrs['class'] = 'form-control'
        html = super(DecimalInput, self).render(name, value, attrs)
        html = html.replace('type="number"', 'type="text"')
        js = "<script>$('#id_{}').mask('###.###.##0,00', {{reverse: true}});</script>".format(name)
        return mark_safe('{}\n{}'.format(html, js))


class MoneyInput(DecimalInput):
    def render(self, name, value, attrs={}):
        html = super(MoneyInput, self).render(name, value, attrs)
        html = '''
        <div class="input-group">
            {}
            <span class="input-group-addon">R$</span>
        </div>
        '''.format(html)
        return mark_safe(html)


class CheckboxInput(widgets.CheckboxInput):
    def render(self, *args, **kwargs):
        html = super(CheckboxInput, self).render(*args, **kwargs)
        html = '{}<span class="custom-checkbox"></span>'.format(html)
        return mark_safe(html)


class RenderableSelectMultiple(widgets.SelectMultiple):
    def __init__(self, template_name='renderable_widget.html', *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)
        self.template_name = template_name

    def render(self, name, value, attrs=None, choices=()):
        if value is None: value = []
        has_id = attrs and 'id' in attrs
        final_attrs = self.build_attrs(attrs, name=name)
        str_values = set([str(v) for v in value])
        i = 0
        objects = []

        qs = hasattr(self.choices.queryset, 'all') and self.choices.queryset.all() or self.choices.queryset
        for obj in qs:
            option_value = obj.pk
            if has_id:
                final_attrs = dict(final_attrs, id='{}_{}'.format(attrs['id'], i))
            final_attrs['class'] = 'custom-checkbox'
            cb = CheckboxInput(final_attrs, check_test=lambda value: value in str_values)
            option_value = str(option_value)
            rendered_cb = cb.render(name, option_value)
            obj.widget = rendered_cb
            i += 1
            objects.append(obj)

        return mark_safe(render_to_string(self.template_name, dict(objects=objects)))


# File Widgets #


class FileInput(widgets.ClearableFileInput):
    def render(self, *args, **kwargs):
        widget = super(FileInput, self).render(*args, **kwargs)

        tokens = widget.split('Modificar: ')
        if len(tokens) == 2:
            extra, input = tokens
            extra = extra.replace('<label ', '<span class="custom-checkbox"></span><label ')
        else:
            extra, input = '', tokens[0]

        input = input[0:-2] + 'class="file-input" ' + input[-2:]
        html = '''
            <div class="upload-file">
                 {}
                 <label for="{}" style="padding:5px"></label>
            </div>
            <div style="margin-top:20px">{}</div>
            <script>$('#{}').change(function(){{$(this).parent().find('label').html($(this).val().split('\\\\').pop());}});</script>
        '''.format(input, kwargs['attrs']['id'], extra, kwargs['attrs']['id'])
        return mark_safe(html)


class ImageInput(widgets.ClearableFileInput):
    def render(self, *args, **kwargs):
        widget = super(ImageInput, self).render(*args, **kwargs)

        tokens = widget.split('Modificar:')
        if len(tokens) == 2:
            extra, input = tokens
            extra = extra.replace('<label ', '<span class="custom-checkbox"></span><label ')
        else:
            extra, input = '', tokens[0]

        input = input[0:-2] + 'class="file-input" ' + input[-2:]
        html = '''
            <div class="upload-file">
                 {}
                 <label for="{}" style="padding:5px"></label>
            </div>
            <div style="margin-top:20px">{}</div>
            <script>$('#{}').change(function(){{$(this).parent().find('label').html($(this).val().split('\\\\').pop());}});</script>

        '''.format(input, kwargs['attrs']['id'], extra, kwargs['attrs']['id'])
        return mark_safe(html)


class PhotoWidget(widgets.TextInput):
    def render(self, name, value, attrs={}):
        attrs.update(style='display:none;')
        html = super(PhotoWidget, self).render(name, value, attrs)
        return render_to_string('widgets/photo_widget.html', dict(name=name, html=html))


# Regional Widgets #

class CpfWidget(MaskWidget):
    def __init__(self):
        super(CpfWidget, self).__init__('000.000.000-00')


class CnpjWidget(MaskWidget):
    def __init__(self):
        super(CnpjWidget, self).__init__('00.000.000/0000-00')


class CpfCnpjWidget(TextInput):

    class Media:
        js = ('/static/js/jquery.mask.min-1.7.7.js',)

    def render(self, name, value=None, attrs={}):
        html = super(CpfCnpjWidget, self).render(name, value, attrs)
        html = '''
                <div class="input-group">
                    {}
                </div>
                <script>
                    var CpfCnpjMaskBehavior = function (val) {{
                        return val.replace(/\D/g, '').length <= 11 ? '000.000.000-009' : '00.000.000/0000-00';
                    }},
                    spOptions = {{
                      clearIfNotMatch: true,
                      onKeyPress: function(val, e, field, options) {{
                          field.mask(CpfCnpjMaskBehavior.apply({}, arguments), options);
                        }}
                    }};

                    $('#id_{}').mask(CpfCnpjMaskBehavior, spOptions);
                </script>
                '''.format(html, name)
        return mark_safe(html)


class CepWidget(MaskWidget):
    def __init__(self):
        super(CepWidget, self).__init__('00.000-000')


class CarPlateWidget(MaskWidget):
    def __init__(self):
        super(CarPlateWidget, self).__init__('AAA-0000')


class PhoneWidget(TextInput):

    class Media:
        js = ('/static/js/jquery.mask.min-1.7.7.js',)

    def render(self, name, value, attrs={}):
        html = super(PhoneWidget, self).render(name, value, attrs)
        html = '''
        <div class="input-group">
            {}
            <span class="input-group-addon"><i class="fa fa-phone"></i></span>
        </div>
        <script>
            var SPMaskBehavior = function (val) {{
              return val.replace(/\D/g, '').length === 11 ? '(00) 00000-0000' : '(00) 0000-00009';
            }},
            spOptions = {{
              clearIfNotMatch: true,
              onKeyPress: function(val, e, field, options) {{
                  field.mask(SPMaskBehavior.apply({{}}, arguments), options);
                }}
            }};

            $('#id_{}').mask(SPMaskBehavior, spOptions);
        </script>
        '''.format(html, name)
        return mark_safe(html)

# Utilitary Fields #


class PasswordInput(widgets.PasswordInput):
    def render(self, name, value, attrs={}):
        attrs['class'] = 'form-control'
        return super(PasswordInput, self).render(name, value, attrs)


class DecimalInput3(DecimalInput):

    def render(self, name, value, attrs={}):
        attrs['class'] = 'form-control'
        html = super(DecimalInput, self).render(name, value, attrs)
        html = html.replace('type="number"', 'type="text"')
        js = "<script>$('#id_{}').mask('#.##0,000', {{reverse: true, clearIfNotMatch: true}});</script>".format(name)
        return mark_safe('{}\n{}'.format(html, js))


class OneDigitValidationInput(TextInput):

    class Media:
        js = ('/static/js/jquery.mask.min-1.7.7.js',)

    def render(self, name, value, attrs={}):
        attrs['class'] = 'form-control'
        html = super(OneDigitValidationInput, self).render(name, value, attrs)
        html = html.replace('type="number"', 'type="text"')
        js = "<script>$('#id_{}').mask('#.##0-0', {{reverse: true, clearIfNotMatch: true}});</script>".format(name)
        return mark_safe('{}\n{}'.format(html, js))


class CreditCardWidget(MaskWidget):
    def __init__(self):
        super(CreditCardWidget, self).__init__('9999 9999 9999 9999')



# -*- coding: utf-8 -*-

# Generated by Django 1.11 on 2017-06-18 11:38


from django.db import migrations
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0010_auto_20170428_1545'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='permission_mapping',
            field=djangoplus.db.models.fields.TextField(default=b'{}', verbose_name='Mapeamento de Permiss\xe3o'),
        ),
    ]



# -*- coding: utf-8 -*-

# Generated by Django 1.10 on 2016-08-23 11:27


import django.contrib.auth.models
from django.db import migrations, models
import django.db.models.deletion
import djangoplus.admin.models
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0008_alter_user_username_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'verbose_name': 'Fun\xe7\xe3o',
                'db_table': 'admin_user_groups',
                'managed': False,
                'verbose_name_plural': 'Fun\xe7\xf5es',
            },
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('name', djangoplus.db.models.fields.CharField(blank=True, max_length=30, verbose_name='Nome')),
                ('username', djangoplus.db.models.fields.CharField(max_length=30, unique=True, verbose_name='Login')),
                ('email', djangoplus.db.models.fields.CharField(blank=True, default=b'', max_length=75, verbose_name='E-mail')),
                ('active', djangoplus.db.models.fields.BooleanField(default=True, verbose_name='Ativo?')),
                ('photo', djangoplus.db.models.fields.ImageField(blank=True, default=b'', null=True, upload_to=b'profiles', verbose_name='Foto')),
            ],
            options={
                'verbose_name': 'Usu\xe1rio',
                'verbose_name_plural': 'Usu\xe1rios',
            },
            managers=[
                ('objects', djangoplus.admin.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name='Settings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('initials', djangoplus.db.models.fields.CharField(default='Django+', max_length=255, verbose_name='Nome')),
                ('name', djangoplus.db.models.fields.CharField(default='Django Plus', max_length=255, verbose_name='Descrição')),
                ('logo', djangoplus.db.models.fields.ImageField(blank=True, default=b'', null=True, upload_to=b'config', verbose_name='Logotipo')),
                ('logo_pdf', djangoplus.db.models.fields.ImageField(blank=True, default=b'', help_text='Imagem sem fundo transparente', null=True, upload_to=b'config', verbose_name='Logotipo para PDF')),
                ('icon', djangoplus.db.models.fields.ImageField(blank=True, default=b'/static/images/blank.png', null=True, upload_to=b'config', verbose_name='\xcdcone')),
                ('background', djangoplus.db.models.fields.ImageField(blank=True, default=b'', upload_to=b'config', verbose_name='Background')),
                ('twitter', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Twitter')),
                ('facebook', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Facebook')),
                ('google', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Google')),
                ('pinterest', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='pinterest')),
                ('linkedin', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='Linkedin')),
                ('rss', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='RSS')),
                ('activated', djangoplus.db.models.fields.BooleanField(default=False, verbose_name='Ativado')),
                ('mail_confirmation', djangoplus.db.models.fields.BooleanField(default=False, verbose_name='Confirma\xe7\xe3o via E-mail')),
                ('admin_activation', djangoplus.db.models.fields.BooleanField(default=False, verbose_name='Ativa\xe7\xe3o pelo Administrador')),
                ('address', djangoplus.db.models.fields.TextField(blank=True, null=True, verbose_name='Endere\xe7o')),
                ('phone_1', djangoplus.db.models.fields.PhoneField(blank=True, max_length=255, null=True, verbose_name='Telefone Principal')),
                ('phone_2', djangoplus.db.models.fields.PhoneField(blank=True, max_length=255, null=True, verbose_name='Telefone Secund\xe1rio')),
                ('email', djangoplus.db.models.fields.CharField(blank=True, max_length=255, null=True, verbose_name='E-mail')),
                ('default_color', djangoplus.db.models.fields.CharField(default='#009688', max_length=255, verbose_name='Cor')),
                ('version', djangoplus.db.models.fields.CharField(max_length=255, verbose_name='Vers\xe3o do Sistema')),
                ('server_address', djangoplus.db.models.fields.CharField(default='http://localhost:8000', max_length=255, verbose_name='Endere\xe7o de Acesso')),
                ('system_email_address', djangoplus.db.models.fields.CharField(default='no-reply@djangoplus.net', max_length=255, verbose_name='E-mail de Notifica\xe7\xe3o')),
            ],
            options={
                'verbose_name': 'Configura\xe7\xe3o',
                'verbose_name_plural': 'Configura\xe7\xf5es',
            },
        ),
        migrations.CreateModel(
            name='Unit',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ascii', djangoplus.db.models.fields.SearchField(blank=True, default=b'', editable=False)),
            ],
            options={
                'verbose_name': 'Unidade',
                'verbose_name_plural': 'Unidades',
            },
        ),
        migrations.CreateModel(
            name='UnitRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('role', djangoplus.db.models.fields.ModelChoiceField(on_delete=django.db.models.deletion.CASCADE, to='admin.Role', verbose_name='Fun\xe7\xe3o')),
                ('unit', djangoplus.db.models.fields.ModelChoiceField(on_delete=django.db.models.deletion.CASCADE, to='admin.Unit', verbose_name='Unidade')),
            ],
            options={
                'verbose_name': 'Fun\xe7\xe3o na Unidade',
                'verbose_name_plural': 'Fun\xe7\xf5es na Unidade',
            },
        ),
        migrations.CreateModel(
            name='Group',
            fields=[
            ],
            options={
                'proxy': True,
            },
            bases=('auth.group',),
            managers=[
                ('objects', django.contrib.auth.models.GroupManager()),
            ],
        ),
        migrations.AddField(
            model_name='settings',
            name='associated_groups',
            field=djangoplus.db.models.fields.MultipleModelChoiceField(blank=True, to='admin.Group', verbose_name='Grupos Associados'),
        ),
        migrations.AddField(
            model_name='user',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups'),
        ),
        migrations.AddField(
            model_name='user',
            name='user_permissions',
            field=models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions'),
        ),
    ]



# -*- coding: utf-8 -*-

import sys
import json
import uuid
from djangoplus.utils.mail import send_mail
from djangoplus.db import models
from django.conf import settings
from djangoplus.decorators import action, meta, subset
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.models import ContentTypeManager
from djangoplus.utils.metadata import get_metadata, getattr2, get_field
from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin, \
    UserManager as DjangoUserManager, Group, Permission


class ContentTypeManager(ContentTypeManager):
    pass


class ContentType(ContentType):
    class Meta:
        proxy = True

    objects = ContentTypeManager()


class Log(models.Model):
    ADD = 1
    EDIT = 2
    DELETE = 3

    OPERATION_CHOICES = [[ADD, 'Cadastro'], [EDIT, 'Edição'], [DELETE, 'Exclusão']]

    content_type = models.ForeignKey(ContentType, verbose_name='Objeto', filter=True)
    operation = models.IntegerField(verbose_name='Operação', choices=OPERATION_CHOICES, filter=True)
    user = models.ForeignKey('admin.User', filter=True)
    date = models.DateTimeField(verbose_name='Data/Hora', auto_now=True, filter=True)
    object_id = models.IntegerField(verbose_name='Identificador', search=True)
    object_description = models.CharField(verbose_name='Descrição do Objeto')
    content = models.TextField(verbose_name='Conteúdo', null=True)

    fieldsets = (
        ('Dados Gerais', {'fields': (
        ('content_type', 'operation'), ('user', 'date'), ('object_id', 'object_description'), 'get_tags')}),
        ('Índices', {'relations': ('logindex_set',)}),
    )

    objects = models.Manager()

    class Meta:
        verbose_name = 'Log'
        verbose_name_plural = 'Logs'
        icon = 'fa-history'
        list_per_page = 25

    def __str__(self):
        return 'Log #{}'.format(self.pk)

    def can_add(self):
        return False

    def can_edit(self):
        return False

    def can_delete(self):
        return False

    def get_action_description(self):
        return ('adiciono', 'editou', 'removeu')[self.operation - 1]

    def get_style(self):
        return ('success', 'info', 'danger')[self.operation - 1]

    def get_icon(self):
        return ('plus', 'pencil', 'trash-o')[self.operation - 1]

    @meta('Tags')
    def get_tags(self):
        return json.loads(self.content)

    def create_indexes(self, instance):
        for log_index in get_metadata(instance.__class__, 'logging', (), iterable=True):
            index_object = getattr2(instance, log_index)
            if index_object:
                index_content_type = ContentType.objects.get_for_model(index_object.__class__)
                LogIndex.objects.create(log=self, content_type=index_content_type, object_id=index_object.pk)


class LogIndex(models.Model):
    log = models.ForeignKey(Log, verbose_name='Log', composition=True)
    content_type = models.ForeignKey('contenttypes.ContentType', verbose_name='Dado')
    object_id = models.IntegerField(verbose_name='Identificador', search=True)

    class Meta:
        verbose_name = 'Index'
        verbose_name_plural = 'Indexes'

    def __str__(self):
        return 'Index #{}'.format(self.pk)


class Organization(models.AsciiModel):
    class Meta:
        verbose_name = 'Organização'
        verbose_name_plural = 'Organizações'

    @classmethod
    def subclass(cls):
        return cls.__subclasses__() and cls.__subclasses__()[0] or None

    def get_units(self):
        unit_subclass = Unit.subclass()
        if unit_subclass:
            for field in get_metadata(unit_subclass, 'fields'):
                if field.remote_field and hasattr(field.remote_field.model, 'organization_ptr'):
                    return field.remote_field.model.objects.filter(**{field.name:self.pk})
        return Unit.objects.none()

    def __str__(self):
        if self.pk == 0:
            return 'Todas'
        else:
            return super(Organization, self).__str__()


class Unit(models.AsciiModel):
    class Meta:
        verbose_name = 'Unidade'
        verbose_name_plural = 'Unidades'

    @classmethod
    def subclass(cls):
        return cls.__subclasses__() and cls.__subclasses__()[0] or None

    def get_organization(self):
        organization_subclass = Organization.subclass()
        if organization_subclass:
            for field in get_metadata(type(self), 'fields'):
                if field.remote_field and field.remote_field.model == organization_subclass:
                    return getattr(self, field.name)
        return None

    def __str__(self):
        if self.pk == 0:
            return 'Todas'
        else:
            return super(Unit, self).__str__()


class UserManager(DjangoUserManager):

    def all(self, *args, **kwargs):
        return super(UserManager, self).all()

    def get_queryset(self):
        return models.QuerySet(self.model, using=self._db)

    @subset('Ativos')
    def active(self):
        return self.filter(active=True)

    @subset('Inativos')
    def inactive(self):
        return self.filter(active=False)

    def create_user(self, username, email, password=None, is_superuser=False):
        if not username:
            raise ValueError('The given username must be set')
        email = self.normalize_email(email)
        user = self.model(username=username, email=email, active=True, is_superuser=is_superuser,
                          last_login=None)
        if password:
            user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, username, email, password):
        return self.create_user(username, email, password, True)


class User(AbstractBaseUser, PermissionsMixin):
    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = ['email']

    name = models.CharField('Nome', max_length=30, blank=True, search=True)
    username = models.CharField('Login', max_length=30, unique=True, search=True)
    email = models.CharField('E-mail', max_length=75, blank=True, default='')
    active = models.BooleanField(verbose_name='Ativo?', default=True, filter=True)
    photo = models.ImageField(upload_to='profiles', null=True, blank=True, default='', verbose_name='Foto', exclude=True)

    permission_mapping = models.TextField(verbose_name='Mapeamento de Permissão', default='{}', exclude=True, display=False)
    organization = models.ForeignKey(Organization, verbose_name='Organização', null=True, blank=True, display=False)
    unit = models.ForeignKey(Unit, verbose_name='Unidade', null=True, blank=True, display=False)

    objects = UserManager()

    fieldsets = (
        ('Identificação', {'fields': (('name', 'email'),)}),
        ('Acesso', {'fields': (('username', 'is_superuser'), ('active',))}),
        ('Funções', {'relations': ('role_set',)}),
        ('Mapeamento de Permissão', {'fields': (('organization', 'unit'), 'permission_mapping')}),
    )

    class Meta():
        verbose_name = 'Usuário'
        verbose_name_plural = 'Usuários'
        list_display = 'username', 'name', 'groups'
        add_form = 'UserForm'
        can_admin = 'Gerenciador de Usuários'
        icon = 'fa-user'

    def save(self, *args, **kwargs):
        if not self.password:
            if settings.DEBUG or 'test' in sys.argv:
                password = settings.DEFAULT_PASSWORD
            else:
                password = uuid.uuid4().get_hex()
            self.set_password(password)
        super(User, self).save(*args, **kwargs)

    def __str__(self):
        return self.name or self.username

    @action('Alterar Senha', input='ChangePasswordForm', inline=True)
    def change_password(self, new_password, confirm_password):
        self.set_password(new_password)
        self.save()

    @action('Enviar Convite de Acesso', inline=True, condition='email', category=None)
    def send_access_invitation(self):
        project_name = Settings.default().initials
        subject = 'Acesso ao Sistema - "{}"'.format(project_name)
        message = '''Você está cadastro no sistema <b>{}</b>.
            Para (re)definir sua senha de acesso, clique no botão abaixo.
        '''.format(project_name)
        actions = [('Acessar agora!', '/admin/password/{}/{}/'.format(self.pk, self.password.encode('hex')))]
        send_mail(subject, message, self.email, actions=actions)

    def units(self, group_name=None):
        qs = self.role_set.all()
        if group_name:
            qs = qs.filter(group__name=group_name)
        return qs.values_list('units', flat=True)

    def in_group(self, *group_names):
        return self.role_set.filter(group__name__in=group_names).exists()

    def in_other_group(self, group_name):
        return self.is_superuser or self.role_set.exclude(group__name=group_name).exists()

    def get_permission_mapping(self, model, obj=None):
        from djangoplus.cache import loader
        import json
        permission_mapping = json.loads(self.permission_mapping or '{}')
        permission_mapping_key = obj and '{}:{}'.format(model.__name__, type(obj).__name__) or model.__name__
        if not settings.DEBUG and permission_mapping_key in permission_mapping:
            return permission_mapping[permission_mapping_key]

        organization_lookups = []
        unit_lookups = []
        role_lookups = dict()
        lookups = dict(list_lookups=[], edit_lookups=[], delete_lookups=[])

        for lookup in get_metadata(model, 'list_lookups', (), iterable=True):
            field = get_field(model, lookup)
            if hasattr(field.remote_field.model, 'organization_ptr') or hasattr(field.remote_field.model, 'unit_ptr'):
                if hasattr(field.remote_field.model, 'organization_ptr'):
                    organization_lookups.append(lookup)
                if hasattr(field.remote_field.model, 'unit_ptr'):
                    unit_lookups.append(lookup)
            else:
                role_username = get_metadata(field.remote_field.model, 'role_username')
                if role_username:
                    role_lookups[get_metadata(field.remote_field.model, 'verbose_name')] = '{}__{}'.format(lookup, role_username)
                for subclass in field.remote_field.model.__subclasses__():
                    role_username = get_metadata(subclass, 'role_username')
                    if role_username:
                        role_lookups[get_metadata(subclass, 'verbose_name')] = '{}__{}__{}'.format(
                            lookup, subclass.__name__.lower(), role_username)

        if hasattr(model, 'organization_ptr') and 'id' not in organization_lookups:
            organization_lookups.append('id')

        if hasattr(model, 'unit_ptr') and 'id' not in unit_lookups:
            unit_lookups.append('id')

        if get_metadata(model, 'role_username') and 'id' not in role_lookups:
            role_lookups[get_metadata(model, 'verbose_name')] = get_metadata(model, 'role_username')

        for field in get_metadata(model, 'fields'):
            if field.remote_field and field.remote_field.model:
                if field.remote_field.model in loader.role_models:
                    role_lookups[get_metadata(field.remote_field.model, 'verbose_name')] = '{}__{}'.format(field.name, loader.role_models[field.remote_field.model]['username_field'])
                if field.remote_field.model in loader.abstract_role_models:
                    for to in loader.abstract_role_models[field.remote_field.model]:
                        role_lookups[get_metadata(to, 'verbose_name')] = '{}__{}__{}'.format(
                            field.name, to.__name__.lower(), loader.role_models[to])
                if hasattr(field.remote_field.model, 'unit_ptr_id') and field.name not in unit_lookups:
                    unit_lookups.append(field.name)
                if hasattr(field.remote_field.model, 'organization_ptr_id') and field.name not in organization_lookups:
                    organization_lookups.append(field.name)

        for organization_lookup in organization_lookups:
            if loader.unit_model:
                if organization_lookup == 'id':
                    unit_lookup = loader.unit_model.__name__.lower()
                else:
                    unit_lookup = '{}__{}'.format(organization_lookup, loader.unit_model.__name__.lower())
                if unit_lookup not in unit_lookups and not hasattr(model, 'unit_ptr'):
                    unit_lookups.append(unit_lookup)

        for unit_lookup in unit_lookups:
            if loader.organization_model:
                if unit_lookup == 'id':
                    organization_lookup = loader.organization_model.__name__.lower()
                else:
                    organization_lookup = '{}__{}'.format(unit_lookup, loader.organization_model.__name__.lower())
                if organization_lookup not in organization_lookups and not hasattr(model, 'organization_ptr'):
                    organization_lookups.append(organization_lookup)

        groups = dict()
        for group_name, organization_id, unit_id in self.role_set.values_list('group__name', 'organizations', 'units'):
            if group_name not in groups:
                groups[group_name] = dict(username_lookups=[], organization_ids=[], unit_ids=[])
            if group_name in role_lookups:
                groups[group_name]['username_lookups'].append(role_lookups[group_name])
            if organization_id and not self.unit_id and (organization_id == self.organization_id or not self.organization_id):
                groups[group_name]['organization_ids'].append(organization_id)
            if self.unit_id or unit_id:
                groups[group_name]['unit_ids'].append(self.unit_id or unit_id)

        if model in loader.permissions_by_scope:
            for group_name in groups:

                username_lookups = groups[group_name]['username_lookups']
                unit_ids = list(set(groups[group_name]['unit_ids']))
                organization_ids = list(set(groups[group_name]['organization_ids']))

                can_view = can_view_by_role = can_view_by_unit = can_view_by_organization = False

                if obj:

                    if type(obj) in loader.permissions_by_scope:
                        can_view = group_name in loader.permissions_by_scope[type(obj)].get('add', [])
                        can_view_by_unit = group_name in loader.permissions_by_scope[type(obj)].get('add_by_unit', [])
                        can_view_by_organization = group_name in loader.permissions_by_scope[type(obj)].get('add_by_organization', [])

                if (can_view or can_view_by_role or can_view_by_unit or can_view_by_organization) is False:
                    can_view = group_name in loader.permissions_by_scope[model].get('view', [])
                    can_view_by_role = group_name in loader.permissions_by_scope[model].get('view_by_role', [])
                    can_view_by_unit = group_name in loader.permissions_by_scope[model].get('view_by_unit', [])
                    can_view_by_organization = group_name in loader.permissions_by_scope[model].get('view_by_organization', [])

                if can_view:
                    lookups['list_lookups'] = []
                else:
                    if can_view_by_role:
                        for username_lookup in username_lookups:
                            lookups['list_lookups'].append((username_lookup, (self.username,)))
                    if can_view_by_unit or self.unit_id:
                        if unit_ids and 0 not in unit_ids:
                            for unit_lookup in unit_lookups:
                                lookups['list_lookups'].append(('{}'.format(unit_lookup), unit_ids))
                    if can_view_by_organization:
                        if organization_ids and 0 not in organization_ids:
                            for organization_lookup in organization_lookups:
                                lookups['list_lookups'].append(('{}'.format(organization_lookup), organization_ids))

                can_edit = group_name in loader.permissions_by_scope[model].get('edit', [])
                can_edit_by_role = group_name in loader.permissions_by_scope[model].get('edit_by_role', [])
                can_edit_by_unit = group_name in loader.permissions_by_scope[model].get('edit_by_unit', [])
                can_edit_by_organization = group_name in loader.permissions_by_scope[model].get('edit_by_organization', [])

                if can_edit:
                    lookups['edit_lookups'] = None
                else:
                    if can_edit_by_role:
                        for username_lookup in username_lookups:
                            lookups['edit_lookups'].append((username_lookup, (self.username,)))
                    if can_edit_by_unit and unit_ids:
                        for unit_lookup in unit_lookups:
                            lookups['edit_lookups'].append((unit_lookup, unit_ids))
                    if can_edit_by_organization and organization_ids:
                        for organization_lookup in organization_lookups:
                            lookups['edit_lookups'].append((organization_lookup, organization_ids))

                can_delete = group_name in loader.permissions_by_scope[model].get('delete', [])
                can_delete_by_role = group_name in loader.permissions_by_scope[model].get('delete_by_role', [])
                can_delete_by_unit = group_name in loader.permissions_by_scope[model].get('delete_by_unit', [])
                can_delete_by_organization = group_name in loader.permissions_by_scope[model].get('delete_by_organization', [])

                if can_delete:
                    lookups['delete_lookups'] = None
                else:
                    if can_delete_by_role:
                        for username_lookup in username_lookups:
                            lookups['delete_lookups'].append((username_lookup, (self.username,)))
                    if can_delete_by_unit and unit_ids:
                        for unit_lookup in unit_lookups:
                            lookups['delete_lookups'].append((unit_lookup, unit_ids))
                    if can_delete_by_organization and organization_ids:
                        for organization_lookup in organization_lookups:
                            lookups['delete_lookups'].append((organization_lookup, organization_ids))

                for actions_dict in (loader.actions, loader.class_actions):
                    for category in actions_dict.get(model, ()):
                        for key in list(actions_dict[model][category].keys()):
                            execute_lookups = []
                            view_name = actions_dict[model][category][key]['view_name']
                            can_execute = group_name in loader.permissions_by_scope[model].get('{}'.format(view_name), [])
                            can_execute_by_role = group_name in loader.permissions_by_scope[model].get('{}_by_role'.format(view_name), [])
                            can_execute_by_unit = group_name in loader.permissions_by_scope[model].get('{}_by_unit'.format(view_name), [])
                            can_execute_by_organization = group_name in loader.permissions_by_scope[model].get('{}_by_organization'.format(view_name), [])
                            if can_execute:
                                execute_lookups = None
                            else:
                                if can_execute_by_role:
                                    for username_lookup in username_lookups:
                                        execute_lookups.append((username_lookup, (self.username,)))
                                if can_execute_by_unit and unit_ids:
                                    for unit_lookup in unit_lookups:
                                        execute_lookups.append((unit_lookup, unit_ids))
                                if can_execute_by_organization and organization_ids:
                                    for organization_lookup in organization_lookups:
                                        execute_lookups.append((organization_lookup, organization_ids))
                            if execute_lookups:
                                if view_name not in lookups:
                                    lookups[view_name] = []
                                lookups[view_name] += execute_lookups

        for codename in ('view', 'list'):
            if model in loader.permissions_by_scope:
                if loader.permissions_by_scope[model].get('{}_by_unit'.format(codename)) and not unit_lookups:
                    raise Exception('A "lookup" meta-attribute must point to a Unit model in {}'.format(model))
                if loader.permissions_by_scope[model].get('{}_by_organization'.format(codename)) and (not organization_lookups and not unit_lookups):
                    raise Exception('A "lookup" meta-attribute must point to a Unit or Organization model in {}'.format(model))
                if loader.permissions_by_scope[model].get('{}_by_role'.format(codename)) and not role_lookups:
                    raise Exception('A "lookup" meta-attribute must point to a role model in {}'.format(model))

        permission_mapping[permission_mapping_key] = lookups
        self.permission_mapping = json.dumps(permission_mapping)
        self.save()

        return permission_mapping[permission_mapping_key]

    # gieve a set of group or permission names, this method returns the groups the user belongs to
    def find_groups(self, perm_or_group, exclude=None):
        qs = self.groups.all()
        if perm_or_group:
            permissions = []
            groups = []
            for item in perm_or_group:
                if '.' in item:
                    permissions.append(item.split('.')[1])
                else:
                    groups.append(item)
            if permissions:
                qs = qs.filter(permissions__codename__in=permissions)
            if groups:
                qs = qs.filter(name__in=groups)

        if exclude:
            qs = qs.exclude(name=exclude)
        return qs

    def email_user(self, subject, message, from_email=None, **kwargs):
        send_mail(subject, message, from_email, [self.email], **kwargs)


def role_list_display():
    from djangoplus.cache import loader
    list_display = ['user', 'group']
    if loader.organization_model:
        list_display.append('get_organizations')
    if loader.unit_model:
        list_display.append('get_units')
    return list_display


class Role(models.Model):
    user = models.ForeignKey('admin.User', verbose_name='Usuário', composition=True)
    group = models.ForeignKey('admin.Group', verbose_name='Grupo')
    organizations = models.ManyToManyField(Organization, through='admin.OrganizationRole', verbose_name='Organizações', exclude=True, blank=True)
    units = models.ManyToManyField(Unit, through='admin.UnitRole', verbose_name='Unidades', exclude=True, blank=True)

    fieldsets = (
        ('Dados Gerais', {'fields': ('user', 'group')}),
        ('Organizações', {'fields': ('organizations',)}),
        ('Unidades', {'fields': ('units',)}),
    )

    objects = models.Manager()

    class Meta:
        verbose_name = 'Função'
        verbose_name_plural = 'Funções'
        db_table = 'admin_user_groups'
        managed = False
        list_display = role_list_display
        can_admin = 'Gerenciador de Usuários'

    def __str__(self):
        return '{}'.format(self.group)

    @meta('Organizações')
    def get_organizations(self):
        return self.organizations.all()

    @meta('Unidades')
    def get_units(self):
        return self.units.all()

    def can_edit(self):
        return False

    def can_add(self):
        return True

    def can_define_organizations(self):
        return Organization.objects.exclude(pk=0).exists()

    def can_define_roles(self):
        return Unit.objects.exclude(pk=0).exists()

    @action('Definir Organizações', condition='can_define_organizations', message='Organização(ões) definida(s) com sucesso.', inline=True)
    def define_organizations(self, organizations):
        self.organizationrole_set.all().delete()
        for organization in organizations:
            OrganizationRole.objects.get_or_create(role=self, organization=organization)

    @action('Definir Unidades', condition='can_define_roles', message='Unidade(s) definida(s) com sucesso.', inline=True)
    def define_units(self, units):
        self.unitrole_set.all().delete()
        for unit in units:
            UnitRole.objects.get_or_create(role=self, unit=unit)


class OrganizationRole(models.Model):
    role = models.ForeignKey(Role, verbose_name='Função', composition=True)
    organization = models.ForeignKey(Organization, verbose_name='Organização')

    class Meta:
        verbose_name = 'Papel na Organização'
        verbose_name_plural = 'Papeis na Organização'

    def __str__(self):
        return '{} - {}'.format(self.role, self.organization)


class UnitRole(models.Model):
    role = models.ForeignKey(Role, verbose_name='Função', composition=True)
    unit = models.ForeignKey(Unit, verbose_name='Unidade')

    class Meta:
        verbose_name = 'Função na Unidade'
        verbose_name_plural = 'Funções na Unidade'

    def __str__(self):
        return '{} - {}'.format(self.role, self.unit)


class PermissionManager(models.Manager):

    def get_queryset(self):
        app_labels = []
        for app_label in settings.INSTALLED_APPS:
            if '.' not in app_label:
                app_labels.append(app_label)
        return super(PermissionManager, self).get_queryset().filter(content_type__app_label__in=app_labels)

    # @action('Give to Users')
    def give_to_users(self):
        pass


class Permission(Permission):
    class Meta:
        verbose_name = 'Permissão'
        verbose_name_plural = 'Permissões'
        proxy = True
        icon = 'fa-check'

    fieldsets = (
        ('Dados Gerais', {'fields': ('name',)}),
        ('Usuários', {'relations': ('user_set',)}),
    )

    objects = PermissionManager()
setattr(Permission._meta, 'search_fields', ['codename', 'name'])
setattr(Permission._meta, 'list_filter', ['content_type'])


class Group(Group):
    class Meta:
        verbose_name = 'Grupo'
        verbose_name_plural = 'Grupos'
        proxy = True
        icon = 'fa-users'
        add_form = 'GroupForm'

    fieldsets = (
        ('Dados Gerais', {'fields': ('name',)}),
        ('Permissões::Permissões', {'relations': ('permissions',)}),
        ('Usuários::Usuários', {'relations': ('user_set',)}),
    )

    objects = models.Manager()


class Settings(models.Model):

    class Meta:
        verbose_name = 'Configuração'
        verbose_name_plural = 'Configurações'

    fieldsets = (
        ('Configuração Geral', {'fields': (('initials', 'name'), ('logo', 'logo_pdf'), ('icon', 'background'))}),
        ('Social', {'fields': (('twitter', 'facebook'), ('google', 'pinterest'), ('linkedin', 'rss'))}),
        ('Direitos Autorais', {'fields': ('company', ('phone_1', 'phone_2'), 'address', 'email')}),
        ('Aparência', {'fields': ('default_color',)}),
        ('Servidor', {'fields': (('server_address', 'system_email_address'),)}),
        ('Versão', {'fields': ('version',)}),
    )

    # Application
    initials = models.CharField('Nome', default='Django+')
    name = models.CharField('Descrição', default='Django Plus')
    logo = models.ImageField('Logotipo', upload_to='config', null=True, blank=True, default='')
    logo_pdf = models.ImageField('Logotipo para PDF', upload_to='config', help_text='Imagem sem fundo transparente',
                                 null=True, blank=True, default='')
    icon = models.ImageField('Ícone', upload_to='config', null=True, blank=True)
    background = models.ImageField('Background', upload_to='config', default='', blank=True)

    # Social params
    twitter = models.CharField('Twitter', null=True, blank=True)
    facebook = models.CharField('Facebook', null=True, blank=True)
    google = models.CharField('Google', null=True, blank=True)
    pinterest = models.CharField('pinterest', null=True, blank=True)
    linkedin = models.CharField('Linkedin', null=True, blank=True)
    rss = models.CharField('RSS', null=True, blank=True)

    # Company
    company = models.CharField('Empresa', null=True, blank=True)
    address = models.TextField('Endereço', null=True, blank=True)
    phone_1 = models.PhoneField('Telefone Principal', null=True, blank=True)
    phone_2 = models.PhoneField('Telefone Secundário', null=True, blank=True)
    email = models.CharField('E-mail', null=True, blank=True)

    # Server configuration
    version = models.CharField('Versão do Sistema', exclude=True)
    server_address = models.CharField('Endereço de Acesso', default='http://localhost:8000')
    system_email_address = models.CharField('E-mail de Notificação', default='no-reply@djangoplus.net')

    @staticmethod
    def default():
        qs = Settings.objects.all()
        if qs.exists():
            return qs[0]
        else:
            settings = Settings()
            settings.initials = 'Sistema'
            settings.name = 'Sistema de gerenciamento online, responsivo e multiplataforma'
            settings.twitter = 'https://twitter.com/'
            settings.facebook = 'https://www.facebook.com/'
            settings.google = 'https://plus.google.com/'
            settings.pinterest = 'https://www.pinterest.com/'
            settings.linkedin = 'https://www.linkedin.com/'
            settings.rss = 'https://www.rss.com/'
            settings.company = ''
            settings.address = ''
            settings.phone_1 = ''
            settings.phone_2 = ''
            settings.email = ''
            settings.version = '1.0'
            settings.save()
            return settings



# -*- coding: utf-8 -*-

from os import sep
from djangoplus.conf.base_settings import *
from os.path import abspath, dirname, join, exists

BASE_DIR = abspath(dirname(dirname(__file__)))
PROJECT_NAME = __file__.split(sep)[-2]

STATIC_ROOT = join(BASE_DIR, 'static')
MEDIA_ROOT = join(BASE_DIR, 'media')

DROPBOX_TOKEN = '' # disponível em https://www.dropbox.com/developers/apps
DROPBOX_LOCALDIR = MEDIA_ROOT
DROPBOX_REMOTEDIR = '/'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': join(BASE_DIR, 'sqlite.db'),
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
    }
}

WSGI_APPLICATION = '{}.wsgi.application'.format(PROJECT_NAME)

INSTALLED_APPS += (
    PROJECT_NAME,
    'djangoplus.ui.themes.default',
)

ROOT_URLCONF = '{}.urls'.format(PROJECT_NAME)

if exists(join(BASE_DIR, 'logs')):
    DEBUG = False
    ALLOWED_HOSTS = ['*']

    HOST_NAME = ''
    DIGITAL_OCEAN_TOKEN = ''

    SERVER_EMAIL = 'root@djangoplus.net'
    ADMINS = [('Admin', 'root@djangoplus.net')]

    DROPBOX_TOKEN = ''
    BACKUP_FILES = ['media', 'sqlite.db']
else:
    EMAIL_BACKEND = 'django.core.mail.backends.filebased.EmailBackend'
    EMAIL_FILE_PATH = join(BASE_DIR, 'mail')


EXTRA_JS = ['/static/js/$PROJECT_NAME.js']
EXTRA_CSS = ['/static/css/$PROJECT_NAME.css']




# -*- coding: utf-8 -*-

from django.conf import settings
from django.views.static import serve
from django.conf.urls import include, url
from djangoplus.docs.views import homologate
from djangoplus.docs.views import doc

urlpatterns = [

    url(r'^media/(?P<path>.*)$', serve, {'document_root': settings.MEDIA_ROOT}),
    url(r'^static/(?P<path>.*)$', serve, {'document_root': settings.STATIC_ROOT}),

    url(r'^doc/$', doc),
    url(r'^homologate/$', homologate),

    url(r'', include('djangoplus.ui.components.calendar.urls')),
    url(r'', include('djangoplus.ui.components.breadcrumbs.urls')),
    url(r'', include('djangoplus.ui.components.select.urls')),

    url(r'', include('djangoplus.core.urls')),

]



# -*- coding: utf-8 -*-

from django.conf import settings
from django.utils import translation
from djangoplus.docs.doc import Documentation
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('task', nargs='*', help='...')
        parser.add_argument('--json', action='store_true', dest='json', default=False, help='')

    def handle(self, *args, **options):

        doc = Documentation()
        translation.activate(settings.LANGUAGE_CODE)

        task = (' '.join([x.decode('utf-8') for x in options['task']])).strip()

        if task:
            if task.isdigit():
                index = int(task) - 1
                if len(doc.usecases) > index > 0:
                    usecase = doc.usecases[index]
                    print(str(usecase))
                else:
                    print('There is no task with index {}'.format(task))
            else:
                selected_usecase = None
                for tmp in doc.usecases:
                    if tmp.name == task:
                        selected_usecase = tmp
                if selected_usecase:
                    print(str(selected_usecase))
                else:
                    print('There is no task named {}'.format(task))
        else:
            if options.pop('json', False):
                print(str(doc.as_json()))
            else:
                print(str(doc))


# -*- coding: utf-8 -*-
from cryptography.fernet import Fernet
cipher = Fernet(Fernet.generate_key())


def encrypt(text):
    return cipher.encrypt(text.encode('utf-8')).decode('utf-8')


def decrypt(text):
    return cipher.decrypt(text.encode('utf-8')).decode('utf-8')
